@@ -4,7 +4,7 @@ int main(int argc, char *argv[])
 {
     MCAgency_t agency;
     MCAgencyOptions_t options;
-    int port;
+    int port = 5052;
     int i;
     MC_InitializeAgencyOptions(&options);
     for (i = 0; i < MC_THREAD_ALL; i++) {
