@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -62,7 +62,7 @@
 #include "include/xml_parser.h"
 #include "include/fipa_acl_envelope.h"
 
-#define BACKLOG 10
+#define BACKLOG 200
 
 acc_p
 acc_Initialize(struct mc_platform_s* mc_platform)
@@ -77,6 +77,15 @@ acc_Initialize(struct mc_platform_s* mc_platform)
   acc->waiting_cond = (COND_T*)malloc(sizeof(COND_T));
   COND_INIT(acc->waiting_cond);
 
+	/* Initialize conn_thread stuff */
+	MUTEX_INIT(&acc->conn_thread_lock);
+	COND_INIT(&acc->conn_thread_cond);
+	acc->num_conn_threads = 0;
+
+	/* Initialize msg_thread stuff */
+	MUTEX_INIT(&acc->msg_thread_lock);
+	COND_INIT(&acc->msg_thread_cond);
+	acc->num_msg_threads = 0;
   return acc;
 }
 
@@ -120,6 +129,14 @@ acc_MessageHandlerThread(LPVOID arg)
     }
     if (mc_platform->message_queue->size == 0 && mc_platform->quit)
     {
+			MUTEX_LOCK(&mc_platform->acc->msg_thread_lock);
+			while(mc_platform->acc->num_msg_threads > 0) {
+				COND_WAIT(
+						&mc_platform->acc->msg_thread_cond,
+						&mc_platform->acc->msg_thread_lock
+						);
+			}
+			MUTEX_UNLOCK(&mc_platform->acc->msg_thread_lock);
       MUTEX_UNLOCK(mc_platform->quit_lock);
       MUTEX_UNLOCK(mc_platform->message_queue->lock);
       THREAD_EXIT();
@@ -176,7 +193,9 @@ acc_MessageHandlerThread(LPVOID arg)
             agent_queue_Add(
                 mc_platform->agent_queue,
                 agent);
-          }
+          } else {
+						fprintf(stderr, "agent_Initialize() failed. %s:%d\n", __FILE__, __LINE__);
+					}
           message_Destroy(message);
           /* Send MC_Signal */
           MUTEX_LOCK(mc_platform->MC_signal_lock);
@@ -256,15 +275,13 @@ acc_MessageHandlerThread(LPVOID arg)
         (
          mc_platform, message, mc_platform -> private_key 
         );
-      message_Destroy
-        (
-         message
-        );
     }
   }
   THREAD_EXIT();
 }
 
+
+#define CONN_THREADS 40
 #ifndef _WIN32
   void*
 acc_Thread(void* arg)
@@ -274,24 +291,30 @@ acc_Thread( LPVOID arg )
 #endif
 {
   connection_p connection;
-  message_p message;
-  mtp_http_p mtp_http;
   mc_platform_p mc_platform = (mc_platform_p)arg;
-  fipa_acl_envelope_p fipa_envelope;
-  fipa_acl_message_p fipa_message;
-  fipa_message_string_p fipa_message_string;
   int err;
   int i, j;
-  agent_t* agent;
+
+	connection_thread_arg_t* connection_thread_arg;
+
+	acc_t* acc = mc_platform->acc;
+
+#ifndef _WIN32
+	pthread_attr_t attr;
+	pthread_attr_init(&attr);
+#else
+	int stack_size = 0;
+#endif
+	THREAD_T conn_thread;
+
   /* We want to watch the connectlist for incoming connections, initialize 
    * them into messages, and add the messages to the appropriate queue. */
   while(1) {
     connection = NULL;
-    message = NULL;
-    mtp_http = NULL;
     MUTEX_LOCK(mc_platform->connection_queue->lock);
     MUTEX_LOCK(mc_platform->quit_lock);
-    while (mc_platform->connection_queue->size == 0 && !mc_platform->quit) {
+    while (
+			(mc_platform->connection_queue->size == 0 ) && !mc_platform->quit) {
       MUTEX_UNLOCK(mc_platform->quit_lock);
       COND_WAIT(
           mc_platform->connection_queue->cond,
@@ -302,9 +325,17 @@ acc_Thread( LPVOID arg )
     if 
       (
        mc_platform->connection_queue->size == 0 &&
-       mc_platform->quit
+       mc_platform->quit 
       )
       {
+				MUTEX_LOCK(&mc_platform->acc->conn_thread_lock);
+				while(mc_platform->acc->num_conn_threads > 0) {
+					COND_WAIT(
+						&mc_platform->acc->conn_thread_cond,
+						&mc_platform->acc->conn_thread_lock
+						);
+				}
+				MUTEX_UNLOCK(&mc_platform->acc->conn_thread_lock);
         MUTEX_UNLOCK(mc_platform->quit_lock);
         MUTEX_UNLOCK(mc_platform->connection_queue->lock);
         THREAD_EXIT();
@@ -329,137 +360,206 @@ acc_Thread( LPVOID arg )
 
     /* Continue with normal operation */
     connection = connection_queue_Pop(mc_platform->connection_queue);
-    mtp_http = mtp_http_New();
-	if ( mtp_http_InitializeFromConnection(mtp_http, connection, mc_platform->private_key ) )
-    {
-      connection_Destroy(connection);
-      mtp_http_Destroy(mtp_http);
-      continue;
-    }
-
-    switch(mtp_http->http_performative)
-    {
-      case HTTP_POST:
-      case HTTP_PUT:
-        /* See if the incoming message is going to any of our services. */
-        /* See if it is a message for the AMS */
-        if(
-            !strcmp(mtp_http->target, "/ams") ||
-            !strcmp( strrchr(mtp_http->target, (int)'/'), "/ams" )
-            ) {
-          message = message_New();
-          /*message->message_parts = mtp_http->message_parts;*/
-          message->message_body = (char*)malloc
-            (
-             sizeof(char) * 
-             (strlen((char*)mtp_http->content->data)+1)
-            );
-          strcpy(message->message_body, (char*)mtp_http->content->data);
-          message->xml_root = mxmlLoadString
-            (
-             NULL,
-             message->message_body,
-             MXML_NO_CALLBACK
-            );
-          if(message_xml_parse(message)) {
-            fprintf(stderr, "Error parsing message. %s:%d\n",
-                __FILE__,__LINE__);
-            message_Destroy(message);
-            mtp_http_Destroy(mtp_http);
-            continue;
-          }
-          mtp_http_Destroy(mtp_http);
-          break;
-        } else if 
-          ( 
-           !strcmp(mtp_http->target, "/acc") ||
-           !strcmp( strrchr(mtp_http->target, (int)'/'), "/acc")
-          ) {
-          /* Make sure there are two parts to the html message.
-           * We expect to receive an xml envelope with a message attached. */
-          if (mtp_http->message_parts != 2) {
-            fprintf(stderr, "Error parsing message. %s:%d\n",
-                __FILE__,__LINE__);
-            mtp_http_Destroy(mtp_http);
-            continue;
-          }
-          /* Now we need to parse the envelope */
-          fipa_envelope = fipa_acl_envelope_New();
-          err = fipa_envelope_Parse(fipa_envelope, (char*)mtp_http->content[0].data);
-          if (err) {
-            fprintf(stderr, "Error parsing message. %s:%d\n",
-                __FILE__, __LINE__);
-            fipa_acl_envelope_Destroy(fipa_envelope);
-            mtp_http_Destroy(mtp_http);
-            continue;
-          }
-          /* Now we need to check the envelope receivers and put copies of the message into
-           * each receiving agent's mailbox. */
-          for(i = 0; i < fipa_envelope->num_params; i++) {
-            for(j = 0; j < fipa_envelope->params[i]->to->num; j++) {
-              agent = agent_queue_SearchName(
-                  mc_platform->agent_queue,
-                  fipa_envelope->params[i]->to->fipa_agent_identifiers[j]->name
-                  );
-              if (agent != NULL) {
-                /* Parse, copy, and deliver the message to the agent. */
-                fipa_message_string = fipa_message_string_New();
-                fipa_message_string->message = strdup((char*)mtp_http->content[1].data);
-                fipa_message_string->parse = fipa_message_string->message;
-                fipa_message = fipa_acl_message_New();
-                err = fipa_acl_Parse(fipa_message, fipa_message_string);
-                if (err) {
-                  fipa_message_string_Destroy(fipa_message_string);
-                  fipa_acl_message_Destroy(fipa_message);
-                  fipa_acl_envelope_Destroy(fipa_envelope);
-                  mtp_http_Destroy(mtp_http);
-                  continue;
-                }
-                agent_mailbox_Post( agent->mailbox, fipa_message);
-                fipa_message_string_Destroy(fipa_message_string);
-              }
-            }
-          }
-          fipa_acl_envelope_Destroy(fipa_envelope);
-          mtp_http_Destroy(mtp_http);
-          continue;
-        }
-        else {
-          /* FIXME: We don't support other recipients yet */
-          fprintf(stderr, "Unsupported. %s:%d\n", __FILE__, __LINE__);
-          mtp_http_Destroy(mtp_http);
-        }
-      default:
-        fprintf(stderr, "unsupported http performative. %s:%d\n",
-            __FILE__, __LINE__);
-    }
-
-    /* If we get here, then we have an incoming message for the ams */
-    connection_Destroy(connection);
-    switch(message->message_type) {
-      case RELAY:
-      case REQUEST:
-      case SUBSCRIBE:
-      case CANCEL:
-      case N_UNDRSTD:
-      case MOBILE_AGENT:
-      case QUER_IF:
-      case QUER_REF:
-      case AGENT_UPDATE:
-      case RETURN_MSG:
-      case FIPA_ACL:
-        message_queue_Add(mc_platform->message_queue, message);
-        break;
-      default:
-        fprintf(stderr, "Unknown message type:%d. Rejecting message.%s:%d\n",
-            message->message_type,
-            __FILE__, __LINE__ );
-        free(message);
-        break;
-    }
+		connection_thread_arg = (connection_thread_arg_t*)malloc(sizeof(connection_thread_arg_t));
+		connection_thread_arg->mc_platform = mc_platform;
+		connection_thread_arg->connection = connection;
+		MUTEX_LOCK(&acc->conn_thread_lock);
+		while(acc->num_conn_threads > CONN_THREADS) {
+			COND_WAIT(&acc->conn_thread_cond, &acc->conn_thread_lock);
+		}
+		acc->num_conn_threads++;
+		MUTEX_UNLOCK(&acc->conn_thread_lock);
+		THREAD_CREATE(&conn_thread, acc_connection_Thread, connection_thread_arg);
+		THREAD_DETACH(conn_thread);
   }
 }
 
+
+#define CONNECT_THREAD_EXIT() \
+	free(arg); \
+	MUTEX_LOCK(&acc->conn_thread_lock); \
+	acc->num_conn_threads--; \
+	COND_SIGNAL(&acc->conn_thread_cond); \
+	MUTEX_UNLOCK(&acc->conn_thread_lock); \
+	THREAD_EXIT();
+
+#ifndef _WIN32
+  void*
+acc_connection_Thread(void* arg)
+#else
+  DWORD WINAPI
+acc_connection_Thread( LPVOID arg )
+#endif
+{
+  mtp_http_p mtp_http;
+	acc_t* acc;
+	mc_platform_t* mc_platform;
+	connection_t* connection;
+  message_p message;
+  fipa_acl_envelope_p fipa_envelope;
+	int err;
+	int i, j;
+  agent_t* agent;
+  fipa_message_string_p fipa_message_string;
+  fipa_acl_message_p fipa_message;
+
+	mc_platform = ((connection_thread_arg_t*)arg)->mc_platform;
+	acc = ((connection_thread_arg_t*)arg)->mc_platform->acc;
+	connection = ((connection_thread_arg_t*)arg)->connection;
+	mtp_http = mtp_http_New();
+	if ( mtp_http_InitializeFromConnection(mtp_http, connection, mc_platform->private_key ) )
+	{
+		fprintf(stderr, "mtp_http_InitializeFromConnection failed. %s:%d\n", __FILE__, __LINE__);
+		connection_Destroy(connection);
+		mtp_http_Destroy(mtp_http);
+		CONNECT_THREAD_EXIT();
+	}
+
+	switch(mtp_http->http_performative)
+	{
+		case HTTP_POST:
+		case HTTP_PUT:
+			/* See if the incoming message is going to any of our services. */
+			/* See if it is a message for the AMS */
+			if(
+					!strcmp(mtp_http->target, "/ams") ||
+					!strcmp( strrchr(mtp_http->target, (int)'/'), "/ams" )
+				) {
+				message = message_New();
+				/*message->message_parts = mtp_http->message_parts;*/
+				message->message_body = (char*)malloc
+					(
+					 sizeof(char) * 
+					 (strlen((char*)mtp_http->content->data)+1)
+					);
+				strcpy(message->message_body, (char*)mtp_http->content->data);
+				message->xml_root = mxmlLoadString
+					(
+					 NULL,
+					 message->message_body,
+					 MXML_NO_CALLBACK
+					);
+				if (message->xml_root == NULL) {
+					fprintf(stderr, "xml loadstring error. %s:%d\n", __FILE__, __LINE__);
+				}
+				if(message_xml_parse(message)) {
+					fprintf(stderr, "Error parsing message. %s:%d\n",
+							__FILE__,__LINE__);
+					message_Destroy(message);
+					mtp_http_Destroy(mtp_http);
+					CONNECT_THREAD_EXIT();
+				}
+				mtp_http_Destroy(mtp_http);
+				break;
+			} else if 
+				( 
+				 !strcmp(mtp_http->target, "/acc") ||
+				 !strcmp( strrchr(mtp_http->target, (int)'/'), "/acc")
+				) {
+					/* Make sure there are two parts to the html message.
+					 * We expect to receive an xml envelope with a message attached. */
+					if (mtp_http->message_parts != 2) {
+						fprintf(stderr, "Error parsing message. %s:%d\n",
+								__FILE__,__LINE__);
+						mtp_http_Destroy(mtp_http);
+						CONNECT_THREAD_EXIT();
+					}
+					/* Now we need to parse the envelope */
+					fipa_envelope = fipa_acl_envelope_New();
+					err = fipa_envelope_Parse(fipa_envelope, (char*)mtp_http->content[0].data);
+					if (err) {
+						fprintf(stderr, "Error parsing message. %s:%d\n",
+								__FILE__, __LINE__);
+						fipa_acl_envelope_Destroy(fipa_envelope);
+						mtp_http_Destroy(mtp_http);
+						CONNECT_THREAD_EXIT();
+					}
+					/* Now we need to check the envelope receivers and put copies of the message into
+					 * each receiving agent's mailbox. */
+					for(i = 0; i < fipa_envelope->num_params; i++) {
+						char* portstr;
+						for(j = 0; j < fipa_envelope->params[i]->to->num; j++) {
+							agent = agent_queue_SearchName(
+									mc_platform->agent_queue,
+									fipa_envelope->params[i]->to->fipa_agent_identifiers[j]->name
+									);
+							if (agent != NULL) {
+                // we do the following incase we have
+                //   sda@localhost:5050
+                //   sda@localhost:5051
+                //   sda@localhost:5052
+                // we don't want to post 3 times since agent_queue_SearchName
+                //   will always return true
+                // FIXME: A better way may be to make sure the port numbers match
+                // 
+                // we assume the addres is http://###.###.###.###:PORT/acc
+                // we get the second ":" and then let atoi() handle the rest
+                portstr = fipa_envelope->params[i]->to->fipa_agent_identifiers[j]->addresses->urls[0]->str;
+                portstr = strstr(portstr, ":")+1;
+                portstr = strstr(portstr, ":")+1;
+                //agent->home_port is never set???
+                if(agent->mc_platform->port != atoi(portstr)) continue;
+ 
+								/* Parse, copy, and deliver the message to the agent. */
+								fipa_message_string = fipa_message_string_New();
+								fipa_message_string->message = strdup((char*)mtp_http->content[1].data);
+								fipa_message_string->parse = fipa_message_string->message;
+								fipa_message = fipa_acl_message_New();
+								err = fipa_acl_Parse(fipa_message, fipa_message_string);
+								if (err) {
+									fipa_message_string_Destroy(fipa_message_string);
+									fipa_acl_message_Destroy(fipa_message);
+									fipa_acl_envelope_Destroy(fipa_envelope);
+									mtp_http_Destroy(mtp_http);
+									CONNECT_THREAD_EXIT();
+								}
+								agent_mailbox_Post( agent->mailbox, fipa_message);
+								fipa_message_string_Destroy(fipa_message_string);
+							}
+						}
+					}
+					fipa_acl_envelope_Destroy(fipa_envelope);
+					mtp_http_Destroy(mtp_http);
+					CONNECT_THREAD_EXIT();
+				}
+			else {
+				/* FIXME: We don't support other recipients yet */
+				fprintf(stderr, "Unsupported. %s:%d\n", __FILE__, __LINE__);
+				mtp_http_Destroy(mtp_http);
+			}
+		default:
+			fprintf(stderr, "unsupported http performative. %s:%d\n",
+					__FILE__, __LINE__);
+	}
+
+	/* If we get here, then we have an incoming message for the ams */
+	connection_Destroy(connection);
+	switch(message->message_type) {
+		case RELAY:
+		case REQUEST:
+		case SUBSCRIBE:
+		case CANCEL:
+		case N_UNDRSTD:
+		case MOBILE_AGENT:
+		case QUER_IF:
+		case QUER_REF:
+		case AGENT_UPDATE:
+		case RETURN_MSG:
+		case FIPA_ACL:
+			message_queue_Add(mc_platform->message_queue, message);
+			break;
+		default:
+			fprintf(stderr, "Unknown message type:%d. Rejecting message.%s:%d\n",
+					message->message_type,
+					__FILE__, __LINE__ );
+			free(message);
+			break;
+	}
+	CONNECT_THREAD_EXIT();
+}
+#undef CONNECT_THREAD_EXIT
+
+
   void
 acc_Start(mc_platform_p mc_platform)
 {
@@ -501,6 +601,12 @@ acc_Start(mc_platform_p mc_platform)
      listen_Thread,
      mc_platform 
     );
+  THREAD_CREATE
+    (
+     &acc->udplisten_thread,
+     udplisten_Thread,
+     mc_platform 
+    );
 }
 
 int
@@ -581,6 +687,9 @@ listen_Thread( LPVOID arg )
 
   /* Set up the socket */
   sockfd = socket(PF_INET, SOCK_STREAM, 0);
+	if (sockfd < 0) {
+		SOCKET_ERROR();
+	}
   mc_platform->sockfd = sockfd;
   sktin.sin_family = AF_INET;
   sktin.sin_port = htons(mc_platform->port);
@@ -651,8 +760,7 @@ listen_Thread( LPVOID arg )
           MUTEX_UNLOCK(mc_platform->acc->waiting_lock);
           /* create a new connection and insert it into the connection linked 
            * list */
-          connection = (connection_p)malloc(sizeof(connection_t));
-          CHECK_NULL(connection, exit(0););
+          connection = connection_New();
           connection->connect_id = rand();
           connection->remote_hostname = NULL;
           connection->addr = peer_addr;
@@ -676,3 +784,61 @@ listen_Thread( LPVOID arg )
   /* Free the current thread */
   THREAD_EXIT();
 }
+
+#define BUFLEN 512
+#define UDPPORT 8866
+#ifndef _WIN32
+  void*
+udplisten_Thread(void* arg)
+#else
+  DWORD WINAPI
+udplisten_Thread( LPVOID arg )
+#endif
+{
+  mc_platform_p mc_platform = (mc_platform_p)arg;
+	struct sockaddr_in si_me, si_remote;
+	int s, i, slen = sizeof(si_remote);
+	char buf[BUFLEN];
+	char mc_req_string[] = "Mobile-C Agency Information Request";
+
+	if((s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1 ) {
+		perror("Socket error.");	
+		exit(1);
+	}
+
+	memset((char*) &si_me, 0, sizeof(si_me));
+	si_me.sin_family = AF_INET;
+	si_me.sin_port = htons(UDPPORT);
+	si_me.sin_addr.s_addr = htons(INADDR_ANY); 
+
+	if(bind(s, (const struct sockaddr*)&si_me, sizeof(si_me)) == -1) {
+		/* Some other agency is already on this host listening. Just disable the udp thread for now. */
+		return NULL;
+	}
+	while(1) {
+		if (recvfrom(s, buf, BUFLEN, 0, (struct sockaddr*)&si_remote, &slen)==-1){
+			perror("recvfrom");
+			continue;
+		}
+		/* Received a udp packet. Let's inspect the contents. */
+		/* It should read :
+		   Mobile-C Agency Information Request
+			 Version <Sending-Agency's-Version>
+		*/
+
+    if(strncmp(buf, mc_req_string, strlen(mc_req_string)) == 0) {
+			sprintf(buf, "Mobile-C Version %s\n%s:%d\n", 
+			PACKAGE_VERSION, mc_platform->hostname, mc_platform->port);
+			if(sendto(s, buf, strlen(buf)+1, 0, (const struct sockaddr*)&si_remote, slen) == -1) {
+				perror("sendto");
+			}
+		} else {
+			sprintf(buf, "Message not understood.");
+			if(sendto(s, buf, strlen(buf)+1, 0, (const struct sockaddr*)&si_remote, slen) == -1) {
+				perror("sendto");
+			}
+		}
+	}
+}
+#undef BUFLEN
+#undef UDPPORT
