@@ -1,6 +1,6 @@
  /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -61,7 +61,7 @@ int agent_AddPersistentVariable(agent_p agent, int task_num, const char* var_nam
 
   /* Get the array data type */
   agent_variable_data->data_type = Ch_DataType(
-      agent->agent_interp,
+      *agent->agent_interp,
       var_name );
   /* Make sure the variable exists */
   if (agent_variable_data->data_type == CH_UNDEFINETYPE) {
@@ -72,7 +72,7 @@ int agent_AddPersistentVariable(agent_p agent, int task_num, const char* var_nam
   }
   /* Get the array dimension */
   agent_variable_data->array_dim = Ch_ArrayDim(
-      agent->agent_interp,
+      *agent->agent_interp,
       var_name );
   /* Get the array extents */
   agent_variable_data->array_extent = (int*)malloc(
@@ -80,7 +80,7 @@ int agent_AddPersistentVariable(agent_p agent, int task_num, const char* var_nam
   for (i=0; i<agent_variable_data->array_dim; i++) {
     agent_variable_data->array_extent[i] = 
       Ch_ArrayExtent(
-          agent->agent_interp,
+          *agent->agent_interp,
           var_name,
           i );
   }
@@ -104,7 +104,7 @@ int agent_AddPersistentVariable(agent_p agent, int task_num, const char* var_nam
     memcpy(
         agent_variable_data->data,
         (void*)Ch_GlobalSymbolAddrByName(
-          agent->agent_interp,
+          *agent->agent_interp,
           var_name),
         size*data_type_size
         );
@@ -113,7 +113,7 @@ int agent_AddPersistentVariable(agent_p agent, int task_num, const char* var_nam
     memcpy(
         agent_variable_data->data,
         (void*)Ch_GlobalSymbolAddrByName(
-          agent->agent_interp,
+          *agent->agent_interp,
           var_name),
         size*data_type_size
         );
@@ -187,6 +187,9 @@ agent_Copy(const agent_p agent)
   cp_agent->lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
   MUTEX_INIT(cp_agent->lock);
 
+	/* Mailbox */
+	cp_agent->mailbox = agent_mailbox_New();
+
   return cp_agent;
 }
 
@@ -213,6 +216,79 @@ agent_New(void)
   return agent;
 }
 
+  agent_p 
+agent_NewBinary( struct mc_platform_s *mc_platform)
+{
+  agent_p agent; 
+  int err_code;
+
+  /* malloc memory for the agent */
+  agent = (MCAgent_t)malloc(sizeof(agent_t));
+  memset(agent, 0, sizeof(agent_t));
+
+  /* Set up general agent data access mutex */
+  agent->lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
+  MUTEX_INIT(agent->lock);
+
+  /* Set up run_lock mutex */
+  agent->run_lock = (MUTEX_T*)malloc(sizeof(MUTEX_T));
+  MUTEX_INIT(agent->run_lock);
+
+  /* set flags and variables for the agent */
+  agent->id = rand();
+#ifndef _WIN32
+  agent->arrival_time = time(NULL);
+#else
+  GetSystemTime( &(agent->arrival_time) );
+#endif
+
+  agent->home = NULL;
+  agent->sender = NULL;
+
+  /* set flags */
+  agent->orphan = 0;
+  agent->agent_script_ready = 1;
+  agent->agent_pipe_ready_to_read = 0;
+  agent->agent_ready_to_send = 0;
+  agent->agent_pipe_active = 0;
+	agent->binary = 1;
+
+  /* set the agent thread to null until activated */
+  agent->agent_thread_id = 0;
+
+  /* Set up an empty mailbox */
+  agent->mailbox = agent_mailbox_New();
+
+  /* In the future we will compare the current tasks server name to 
+     the one on the server, presently this is not implemented */
+
+  /* set the CH exectution flag */
+  /* FIXME: This should be in the xml parser */
+  /*if (agent->datastate->tasks[agent->datastate->task_progress]->init_agent_status != -1) {
+    agent->agent_status = agent->datastate->tasks[agent->datastate->task_progress]->init_agent_status;
+    } else {
+    agent->agent_status = mc_platform->default_agentstatus;
+    }
+    */
+  agent->agent_status = MC_AGENT_ACTIVE;
+
+  agent->mc_platform = mc_platform;
+
+	agent->agent_address = (char*)malloc(sizeof(char) * 
+			(strlen(agent->mc_platform->hostname) + 12 + 10)
+		);
+	if (agent->agent_address == NULL) {
+		fprintf(stderr, "Memory error. %s:%d\n", __FILE__, __LINE__);
+		exit(-1);
+	}
+  sprintf(agent->agent_address,
+      "http://%s:%d/acc",
+      agent->mc_platform->hostname,
+      agent->mc_platform->port
+      );
+  /* return */
+  return agent;
+}
   agent_p 
 agent_Initialize(
     struct mc_platform_s *mc_platform,
@@ -318,6 +394,18 @@ agent_Initialize(
 
   agent->mc_platform = mc_platform;
 
+	agent->agent_address = (char*)malloc(sizeof(char) * 
+			(strlen(agent->mc_platform->hostname) + 12 + 10)
+		);
+	if (agent->agent_address == NULL) {
+		fprintf(stderr, "Memory error. %s:%d\n", __FILE__, __LINE__);
+		exit(-1);
+	}
+  sprintf(agent->agent_address,
+      "http://%s:%d/acc",
+      agent->mc_platform->hostname,
+      agent->mc_platform->port
+      );
   /* return */
   return agent;
 }
@@ -341,11 +429,15 @@ agent_Destroy(agent_p agent)
   if (agent->sender != NULL) {
     free(agent->sender);
   }
+	if (agent->wg_code != NULL) {
+		free(agent->wg_code);
+	}
   /* Terminate the agent datastate memory */
   MUTEX_DESTROY(agent->lock);
   if (agent->agent_status == MC_AGENT_NEUTRAL) {
     if ((agent->agent_interp) != NULL) {
-      Ch_End(agent->agent_interp);
+      Ch_Reset(*agent->agent_interp);
+			interpreter_queue_Add(agent->mc_platform->interpreter_queue, agent->agent_interp);
     }
   }
   free(agent->lock);
@@ -394,6 +486,436 @@ agent_RunChScript(agent_p agent, mc_platform_p mc_platform)
   return;
 }
 
+
+/* Added for now */
+void *
+agent_ChScriptInitVar(ChInterp_t* interp)
+{
+  char *tmp_buf;
+
+  /* Declare special variables for holding the agent id and name */
+  tmp_buf = malloc(sizeof(char) * 200);
+  tmp_buf[0] = '\0';
+  sprintf(tmp_buf, "int mc_agent_id = -1;");
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  tmp_buf[0] = '\0';
+  //sprintf(tmp_buf, "char mc_agent_name[100] = \"agent_name\";");
+  sprintf(tmp_buf, "char* mc_agent_name = NULL;");
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  tmp_buf[0] = '\0';
+  sprintf(tmp_buf, "void* mc_current_agent = NULL;");
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  tmp_buf[0] = '\0';
+  //sprintf(tmp_buf, "char mc_host_name[100] = \"host_name\";");
+  sprintf(tmp_buf, "char* mc_host_name = NULL;");
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  tmp_buf[0] = '\0';
+  sprintf(tmp_buf, "int mc_host_port = -1;\n");
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  tmp_buf[0] = '\0';
+  sprintf(tmp_buf, "int mc_task_progress = -1;\n");
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  tmp_buf[0] = '\0';
+  sprintf(tmp_buf, "int mc_num_tasks = -1;\n");
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  /* Declare standard error code enum */
+  tmp_buf[0] = '\0';
+  sprintf(tmp_buf, "enum error_code_e {MC_SUCCESS = 0, MC_ERR, MC_ERR_CONNECT, MC_ERR_PARSE, MC_ERR_EMPTY, MC_ERR_INVALID, MC_ERR_INVALID_ARGS, MC_ERR_NOT_FOUND, MC_ERR_MEMORY, MC_ERR_SEND, MC_WARN_DUPLICATE };" );
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  Ch_DeclareVar(
+      *interp,
+      "char* mc_agent_address;"
+      );
+
+
+  tmp_buf[0] = '\0';
+  sprintf(tmp_buf, "enum MC_SteerCommand_e {MC_RUN = 0, MC_SUSPEND, MC_RESTART, MC_STOP};" );
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  tmp_buf[0] = '\0';
+  sprintf(tmp_buf, "enum mc_AgentStatus_e { MC_WAIT_CH, MC_WAIT_MESSGSEND, MC_AGENT_ACTIVE, MC_AGENT_NEUTRAL, MC_AGENT_SUSPENDED, MC_WAIT_FINISHED};"); 
+  Ch_DeclareVar(
+      *interp,
+      tmp_buf
+      );
+
+  free(tmp_buf);
+  /* Add the MCAgent_t typedef */
+  Ch_DeclareVar(
+      *interp,
+      "void* MCAgent_t;"
+      );
+  Ch_DeclareTypedef(
+      *interp,
+      "MCAgent_t"
+      );
+
+  /* Following are the declarations of the agent-space api functions. */
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclDestroy(void* acl_message);",
+      (ChFuncdl_t)MC_AclDestroy_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "void* mc_AclNew(void);",
+      (ChFuncdl_t)MC_AclNew_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclPost(void* agent, void* acl_message);",
+      (ChFuncdl_t)MC_AclPost_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "void* mc_AclRetrieve(void* agent);",
+      (ChFuncdl_t)MC_AclRetrieve_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "void* mc_AclReply(void* acl_message);",
+      (ChFuncdl_t)MC_AclReply_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclSend(void* acl_message);",
+      (ChFuncdl_t)MC_AclSend_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "void* mc_AclWaitRetrieve(void* agent);",
+      (ChFuncdl_t)MC_AclWaitRetrieve_chdl
+      );
+  /* begin Acl Helper Functions */
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclSetProtocol(void* acl_message, int protocol);",
+      (ChFuncdl_t)MC_AclSetProtocol_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclSetConversationID(void* acl_message, char* id);",
+      (ChFuncdl_t)MC_AclSetConversationID_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclSetPerformative(void* acl_message, int performative);",
+      (ChFuncdl_t)MC_AclSetPerformative_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclSetSender(void* acl_message, char* name, char* address);",
+      (ChFuncdl_t)MC_AclSetSender_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclAddReceiver(void* acl_message, char* name, char* address);",
+      (ChFuncdl_t)MC_AclAddReceiver_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclAddReplyTo(void* acl_message, char* name, char* address);",
+      (ChFuncdl_t)MC_AclAddReplyTo_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AclSetContent(void* acl_message, char* content);",
+      (ChFuncdl_t)MC_AclSetContent_chdl
+      );
+  /* end Acl Helper Functions */
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AddAgent(void* agent);",
+      (ChFuncdl_t)MC_AddAgent_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "const void* mc_AgentVariableRetrieve(void* agent, const char* var_name, int task_num);",
+      (ChFuncdl_t)MC_AgentVariableRetrieve_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_AgentVariableSave(void* agent, const char* var_name);",
+      (ChFuncdl_t)MC_AgentVariableSave_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_Barrier(int id);",
+      (ChFuncdl_t)MC_Barrier_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_BarrierDelete(int id);",
+      (ChFuncdl_t)MC_BarrierDelete_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_BarrierInit(int id, int num_procs);",
+      (ChFuncdl_t)MC_BarrierInit_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_CallAgentFunc(char* agentName, const char* funcName, void* returnVal,  ...);",
+      (ChFuncdl_t)MC_CallAgentFunc_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "MCAgent_t mc_ComposeAgent(const char* name, *home, *owner, *code, *return_var_name, *server, *workgroup_code, int persistent);",
+      (ChFuncdl_t)MC_ComposeAgent_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_CondBroadcast(int id);",
+      (ChFuncdl_t)MC_CondBroadcast_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_CondSignal(int id);",
+      (ChFuncdl_t)MC_CondSignal_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_CondReset(int id);",
+      (ChFuncdl_t)MC_CondReset_chdl 
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_CondWait(int id);",
+      (ChFuncdl_t)MC_CondWait_chdl 
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_DeleteAgent(const char* agentName);",
+      (ChFuncdl_t)MC_DeleteAgent_chdl 
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_DeleteAgentWG(MCAgent_t calling_agent, const char* agentName);",
+      (ChFuncdl_t)MC_DeleteAgentWG_chdl 
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_DeregisterService(int agentID, char* serviceName);",
+      (ChFuncdl_t)MC_DeregisterService_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_DestroyServiceSearchResult( char** agentName, char** serviceName, int* agentID, int numResult);",
+      (ChFuncdl_t)MC_DestroyServiceSearchResult_chdl 
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_End(void);",
+      (ChFuncdl_t)MC_End_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "void *mc_FindAgentByID(int id);",
+      (ChFuncdl_t)MC_FindAgentByID_chdl
+      );
+  /* FIXME: This function will probably be deprecated for security reasons */
+  Ch_DeclareFunc(
+      *interp,
+      "void *mc_FindAgentByName(const char *name);",
+      (ChFuncdl_t)MC_FindAgentByName_chdl
+      );
+  /* FIXME -- This block of code does not work: Ch does not
+   * understand 'time_t' */
+  /*
+  Ch_DeclareFunc(
+      *interp,
+      "time_t mc_GetAgentArrivalTime(void* agent);",
+      (ChFuncdl_t)MC_GetAgentArrivalTime_chdl
+      );
+      */
+  Ch_DeclareFunc(
+      *interp,
+      "int MC_GetAgentID(void* agent);",
+      (ChFuncdl_t)MC_GetAgentStatus_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "char* MC_GetAgentName(void* agent);",
+      (ChFuncdl_t)MC_GetAgentStatus_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_GetAgentStatus(void* agent);",
+      (ChFuncdl_t)MC_GetAgentStatus_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "char *mc_GetAgentXMLString(void* agent);",
+      (ChFuncdl_t)MC_GetAgentXMLString_chdl
+      );
+
+#ifndef _WIN32
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_gettimeofday(void* tv);",
+      (ChFuncdl_t)MC_GetTimeOfDay_chdl
+      );
+#endif
+
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_HaltAgency(void);",
+      (ChFuncdl_t)MC_HaltAgency_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_MigrateAgent(void* agent, const char* hostname, int port);",
+      (ChFuncdl_t)MC_MigrateAgent_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_MutexLock(int id);",
+      (ChFuncdl_t)MC_MutexLock_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_MutexUnlock(int id);",
+      (ChFuncdl_t)MC_MutexUnlock_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_PrintAgentCode(void* agent);",
+      (ChFuncdl_t)MC_PrintAgentCode_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_ResumeAgency(void);",
+      (ChFuncdl_t)MC_ResumeAgency_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SearchForService(const char* searchString, char*** agentNames, char*** serviceNames, int** agentIDs, int* numResults);",
+      (ChFuncdl_t)MC_SearchForService_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SendSteerCommand(enum MC_SteerCommand_e command);",
+      (ChFuncdl_t)MC_SendSteerCommand_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_RegisterService(MCAgent_t agent, char **serviceNames, int numServices);",
+      (ChFuncdl_t)MC_RegisterService_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "void *mc_RetrieveAgent(void);",
+      (ChFuncdl_t)MC_RetrieveAgent_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "char *mc_RetrieveAgentCode(void* agent);",
+      (ChFuncdl_t)MC_RetrieveAgentCode_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SaveData(MCAgent_t agent, char* name, int size, void* data);",
+      (ChFuncdl_t)MC_SaveData_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SemaphoreWait(int id);",
+      (ChFuncdl_t)MC_SemaphoreWait_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SemaphorePost(int id);",
+      (ChFuncdl_t)MC_SemaphorePost_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SendAgentMigrationMessage(char *message, char *hostname, int port);",
+      (ChFuncdl_t)MC_SendAgentMigrationMessage_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SendAgentMigrationMessageFile(char *filename, char *hostname, int port);",
+      (ChFuncdl_t)MC_SendAgentMigrationMessageFile_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SetAgentStatus(void* agent, int status);",
+      (ChFuncdl_t)MC_SetAgentStatus_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SetDefaultAgentStatus(int status);",
+      (ChFuncdl_t)MC_SetDefaultAgentStatus_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SyncDelete(int id);",
+      (ChFuncdl_t)MC_SyncDelete_chdl 
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_SyncInit(int id);",
+      (ChFuncdl_t)MC_SyncInit_chdl 
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_TerminateAgent(const char* agentName);",
+      (ChFuncdl_t)MC_TerminateAgent_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_TerminateAgentWG(void* callingAgent, const char* agentName);",
+      (ChFuncdl_t)MC_TerminateAgentWG_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "int mc_GetAgentID(void* agent);",
+      (ChFuncdl_t)MC_GetAgentID_chdl
+      );
+  Ch_DeclareFunc(
+      *interp,
+      "char *mc_GetAgentName(void* agent);",
+      (ChFuncdl_t)MC_GetAgentName_chdl
+      );
+
+  return;
+}
+
 #ifndef _WIN32
   void* 
 agent_RunChScriptThread(void * ChAgent)
@@ -414,6 +936,7 @@ agent_RunChScriptThread(void* ChAgent)
   void *result;
   int progress;
   char *tmp_buf;
+  interpreter_variable_data_t* temp_interp_data;
 
   /* set up the agent object */
   agent = (MCAgent_t)ChAgent;
@@ -430,444 +953,64 @@ agent_RunChScriptThread(void* ChAgent)
     return 0;
 #endif
   }
-  /* We need to check for custom interp_options. If any
-   * value is null, then we pass null on to Ch_Initialize. */
-  if( ((MCAgent_t)ChAgent)->mc_platform->interp_options == NULL ) {
 
-    if(Ch_Initialize(&(((MCAgent_t)ChAgent)->agent_interp), 
-          NULL))
-    {
-      printf("CH INIT ERROR \n");
-      exit(EXIT_FAILURE);
-    }
-  } else {
-    if(Ch_Initialize(&(((MCAgent_t)ChAgent)->agent_interp), 
-          ((MCAgent_t)ChAgent)->mc_platform->interp_options))
-    {
-      printf("CH INIT ERROR \n");
-      exit(EXIT_FAILURE);
-    }
-  }
+  /* Get an interpreter from the list of agency interpreters */
+  agent->agent_interp = interpreter_queue_CreateRetrieve(mc_platform->interpreter_queue,
+		agent->mc_platform->interp_options );
 
-  /* Declare special variables for holding the agent id and name */
-  tmp_buf = malloc(sizeof(char) * 200);
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "int mc_agent_id=%d;", (int)agent->id);
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
+  /* Setup the global variables */
+  //printf("mc_agent_id\n");
+  Ch_SetVar(*agent->agent_interp, "mc_agent_id",   
+      CH_INTTYPE, (int)agent->id);
+  //printf("mc_agent_name\n");
+  Ch_SetVar(*agent->agent_interp, "mc_agent_name", 
+      CH_CHARPTRTYPE, agent->name);
+  //printf("mc_current_agent\n");
+  Ch_SetVar(*agent->agent_interp, "mc_current_agent", 
+      CH_VOIDPTRTYPE, (void*)((size_t)agent)); 
+  //printf("mc_host_name\n");
+  Ch_SetVar(*agent->agent_interp, "mc_host_name", 
+      CH_CHARPTRTYPE, agent->mc_platform->hostname);
+  //printf("mc_host_port\n");
+  Ch_SetVar(*agent->agent_interp, "mc_host_port",   
+      CH_INTTYPE, (int)agent->mc_platform->port);
+  //printf("mc_task_progress\n");
+  Ch_SetVar(*agent->agent_interp, "mc_task_progress",   
+      CH_INTTYPE, (int)agent->datastate->task_progress);
+  //printf("mc_num_tasks\n");
+  Ch_SetVar(*agent->agent_interp, "mc_num_tasks",   
+      CH_INTTYPE, (int)agent->datastate->number_of_tasks);
 
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf,
-      "char mc_agent_name[]=\"%s\";",
-      agent->name
-      );
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
+	Ch_SetVar(*agent->agent_interp, "mc_agent_address",
+			CH_CHARPTRTYPE, agent->agent_address);
 
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "void* mc_current_agent = (void*)%d;", (int)agent);
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
+  /* Originally, we hope to use append runscript if the buffer for code is less 
+     than 51 bytes. Otherwise we must open a file and save the data to the 
+     filename. However, since the mobile agent codes for testing are complete 
+     programs which contain preprocessing directives like "#include" and are 
+     larger than 51 bytes, we use the following statement to save the data to a 
+     file instead of putting it into the buffer. 
 
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "char mc_host_name[] = \"%s\";",
-      agent->mc_platform->hostname );
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
-
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "int mc_host_port = %d;\n",
-      agent->mc_platform->port );
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
-
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "int mc_task_progress = %d;\n",
-      agent->datastate->task_progress);
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
-
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "int mc_num_tasks = %d;\n",
-      agent->datastate->number_of_tasks );
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
-
-  /* Declare standard error code enum */
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "enum error_code_e {MC_SUCCESS = 0, MC_ERR, MC_ERR_CONNECT, MC_ERR_PARSE, MC_ERR_EMPTY, MC_ERR_INVALID, MC_ERR_INVALID_ARGS, MC_ERR_NOT_FOUND, MC_ERR_MEMORY, MC_ERR_SEND, MC_WARN_DUPLICATE };" );
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
-
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "enum MC_SteerCommand_e {MC_RUN = 0, MC_SUSPEND, MC_RESTART, MC_STOP};" );
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
-
-  tmp_buf[0] = '\0';
-  sprintf(tmp_buf, "enum mc_AgentStatus_e { MC_WAIT_CH, MC_WAIT_MESSGSEND, MC_AGENT_ACTIVE, MC_AGENT_NEUTRAL, MC_AGENT_SUSPENDED, MC_WAIT_FINISHED};"); 
-  Ch_DeclareVar(
-      agent->agent_interp,
-      tmp_buf
-      );
-
-  free(tmp_buf);
-  /* Add the MCAgent_t typedef */
-  Ch_DeclareVar(
-      agent->agent_interp,
-      "void* MCAgent_t;"
-      );
-  Ch_DeclareTypedef(
-      agent->agent_interp,
-      "MCAgent_t"
-      );
-
-  /* Following are the declarations of the agent-space api functions. */
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AclDestroy(void* acl_message);",
-      (ChFuncdl_t)MC_AclDestroy_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "void* mc_AclNew(void);",
-      (ChFuncdl_t)MC_AclNew_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "void* mc_AclPost(void* agent, void* acl_message);",
-      (ChFuncdl_t)MC_AclPost_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "void* mc_AclRetrieve(void* agent);",
-      (ChFuncdl_t)MC_AclRetrieve_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "void* mc_AclReply(void* acl_message);",
-      (ChFuncdl_t)MC_AclReply_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AclSend(void* acl_message);",
-      (ChFuncdl_t)MC_AclSend_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "void* mc_AclWaitRetrieve(void* agent);",
-      (ChFuncdl_t)MC_AclWaitRetrieve_chdl
-      );
-  /* begin Acl Helper Functions */
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AclSetPerformative(void* acl_message, int performative);",
-      (ChFuncdl_t)MC_AclSetPerformative_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AclSetSender(void* acl_message, char* name, char* address);",
-      (ChFuncdl_t)MC_AclSetSender_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AclAddReceiver(void* acl_message, char* name, char* address);",
-      (ChFuncdl_t)MC_AclAddReceiver_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AclAddReplyTo(void* acl_message, char* name, char* address);",
-      (ChFuncdl_t)MC_AclAddReplyTo_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AclSetContent(void* acl_message, char* content);",
-      (ChFuncdl_t)MC_AclSetContent_chdl
-      );
-  /* end Acl Helper Functions */
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AddAgent(void* agent);",
-      (ChFuncdl_t)MC_AddAgent_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "const void* mc_AgentVariableRetrieve(void* agent, const char* var_name, int task_num);",
-      (ChFuncdl_t)MC_AgentVariableRetrieve_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_AgentVariableSave(void* agent, const char* var_name);",
-      (ChFuncdl_t)MC_AgentVariableSave_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_Barrier(int id);",
-      (ChFuncdl_t)MC_Barrier_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_BarrierDelete(int id);",
-      (ChFuncdl_t)MC_BarrierDelete_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_BarrierInit(int id, int num_procs);",
-      (ChFuncdl_t)MC_BarrierInit_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_CallAgentFunc(MCAgent_t agent, const char* funcName, void* returnVal,  ...);",
-      (ChFuncdl_t)MC_CallAgentFunc_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "MCAgent_t mc_ComposeAgent(const char* name, *home, *owner, *code, *return_var_name, *server, int persistent);",
-      (ChFuncdl_t)MC_ComposeAgent_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_CondBroadcast(int id);",
-      (ChFuncdl_t)MC_CondBroadcast_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_CondSignal(int id);",
-      (ChFuncdl_t)MC_CondSignal_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_CondReset(int id);",
-      (ChFuncdl_t)MC_CondReset_chdl 
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_CondWait(int id);",
-      (ChFuncdl_t)MC_CondWait_chdl 
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_DeleteAgent(MCAgent_t agent);",
-      (ChFuncdl_t)MC_DeleteAgent_chdl 
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_DeregisterService(int agentID, char* serviceName);",
-      (ChFuncdl_t)MC_DeregisterService_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_DestroyServiceSearchResult( char** agentName, char** serviceName, int* agentID, int numResult);",
-      (ChFuncdl_t)MC_DestroyServiceSearchResult_chdl 
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_End(void);",
-      (ChFuncdl_t)MC_End_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "void *mc_FindAgentByID(int id);",
-      (ChFuncdl_t)MC_FindAgentByID_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "void *mc_FindAgentByName(const char *name);",
-      (ChFuncdl_t)MC_FindAgentByName_chdl
-      );
-  /* FIXME -- This block of code does not work: Ch does not
-   * understand 'time_t' */
-  /*
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "time_t mc_GetAgentArrivalTime(void* agent);",
-      (ChFuncdl_t)MC_GetAgentArrivalTime_chdl
-      );
-      */
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int MC_GetAgentID(void* agent);",
-      (ChFuncdl_t)MC_GetAgentStatus_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "char* MC_GetAgentName(void* agent);",
-      (ChFuncdl_t)MC_GetAgentStatus_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_GetAgentStatus(void* agent);",
-      (ChFuncdl_t)MC_GetAgentStatus_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "char *mc_GetAgentXMLString(void* agent);",
-      (ChFuncdl_t)MC_GetAgentXMLString_chdl
-      );
-
-#ifndef _WIN32
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_gettimeofday(void* tv);",
-      (ChFuncdl_t)MC_GetTimeOfDay_chdl
-      );
-#endif
-
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_HaltAgency(void);",
-      (ChFuncdl_t)MC_HaltAgency_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_MigrateAgent(void* agent, const char* hostname, int port);",
-      (ChFuncdl_t)MC_MigrateAgent_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_MutexLock(int id);",
-      (ChFuncdl_t)MC_MutexLock_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_MutexUnlock(int id);",
-      (ChFuncdl_t)MC_MutexUnlock_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_PrintAgentCode(void* agent);",
-      (ChFuncdl_t)MC_PrintAgentCode_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_ResumeAgency(void);",
-      (ChFuncdl_t)MC_ResumeAgency_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SearchForService(const char* searchString, char*** agentNames, char*** serviceNames, int** agentIDs, int* numResults);",
-      (ChFuncdl_t)MC_SearchForService_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SendSteerCommand(enum MC_SteerCommand_e command);",
-      (ChFuncdl_t)MC_SendSteerCommand_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_RegisterService(MCAgent_t agent, char **serviceNames, int numServices);",
-      (ChFuncdl_t)MC_RegisterService_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "void *mc_RetrieveAgent(void);",
-      (ChFuncdl_t)MC_RetrieveAgent_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "char *mc_RetrieveAgentCode(void* agent);",
-      (ChFuncdl_t)MC_RetrieveAgentCode_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SaveData(MCAgent_t agent, char* name, int size, void* data);",
-      (ChFuncdl_t)MC_SaveData_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SemaphoreWait(int id);",
-      (ChFuncdl_t)MC_SemaphoreWait_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SemaphorePost(int id);",
-      (ChFuncdl_t)MC_SemaphorePost_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SendAgentMigrationMessage(char *message, char *hostname, int port);",
-      (ChFuncdl_t)MC_SendAgentMigrationMessage_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SendAgentMigrationMessageFile(char *filename, char *hostname, int port);",
-      (ChFuncdl_t)MC_SendAgentMigrationMessageFile_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SetAgentStatus(void* agent, int status);",
-      (ChFuncdl_t)MC_SetAgentStatus_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SetDefaultAgentStatus(int status);",
-      (ChFuncdl_t)MC_SetDefaultAgentStatus_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SyncDelete(int id);",
-      (ChFuncdl_t)MC_SyncDelete_chdl 
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_SyncInit(int id);",
-      (ChFuncdl_t)MC_SyncInit_chdl 
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_TerminateAgent(void* agent);",
-      (ChFuncdl_t)MC_TerminateAgent_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "int mc_GetAgentID(void* agent);",
-      (ChFuncdl_t)MC_GetAgentID_chdl
-      );
-  Ch_DeclareFunc(
-      agent->agent_interp,
-      "char *mc_GetAgentName(void* agent);",
-      (ChFuncdl_t)MC_GetAgentName_chdl
-      );
-  /* Originally, we hope to use append runscript if the buffer for code is less than 5120 bytes.
-     Otherwise we must open a file and save the data to the filename.
-     However, since the mobile agent codes for testing are complete programs 
-     which contain preprocessing directives like "#include" and are larger than 51 bytes, we use
-     the following statement to save the data to a file instead of putting it into the buffer. 
-
-FIXME: This is silly
-*/
-  if(strlen(agent->datastate->agent_code) < 51)
+     FIXME: This is silly
+  */
+  if(strlen(agent->datastate->agent_code) < 5)
   {
+    MUTEX_LOCK(agent->run_lock);
     if(Ch_AppendRunScript(
-          ((MCAgent_t)ChAgent)->agent_interp, 
+          *((MCAgent_t)ChAgent)->agent_interp, 
           ((MCAgent_t)ChAgent)->datastate->agent_code))
     {
+      /* DEBUG */
       printf("CH Failure \n");
-      exit(EXIT_FAILURE);
+      printf("***************************************************\nCode was:\n%s\n\n", agent->datastate->agent_code);
+      //exit(EXIT_FAILURE);
     }
-    if(Ch_CallFuncByName(((MCAgent_t)ChAgent)->agent_interp, "main", NULL))
+    if(Ch_CallFuncByName(*((MCAgent_t)ChAgent)->agent_interp, "main", NULL))
     {
       printf("CH2 failure \n");
       exit(EXIT_FAILURE);
-    }
+    } 
+    MUTEX_UNLOCK(agent->run_lock);
   }
   else
   {
@@ -904,13 +1047,15 @@ FIXME: This is silly
     ChShellArg[0] = temp_store_file;
     ChShellArg[1] = NULL;
     MUTEX_LOCK(agent->run_lock);
-    Ch_RunScript(agent->agent_interp, ChShellArg); 
+    if(Ch_RunScript(*agent->agent_interp, ChShellArg) < 0) {
+      fprintf(stderr, "Ch_RunScript error. %s:%d\n", __FILE__, __LINE__);
+    }
 
-    /*        if(Ch_CallFuncByName(((MCAgent_t)ChAgent)->agent_interp, "main", NULL))
-              {
-              printf("CH2 failure \n");
-              exit(EXIT_FAILURE);
-              } */
+    /* if(Ch_CallFuncByName(((MCAgent_t)ChAgent)->agent_interp, "main", NULL))
+       {
+         printf("CH2 failure \n");
+         exit(EXIT_FAILURE);
+       } */
 
     /* remove the temp file after its usage */
     remove(temp_store_file);
@@ -939,7 +1084,17 @@ FIXME: This is silly
 
   /* This is where we want to save all of the agent variables that the agent
    * wishes to keep for its trip. */
+  while (
+	temp_interp_data = agent_variable_list_Pop(
+		agent->datastate->tasks[progress]->agent_variable_list )
+	) {
+	interpreter_variable_data_Destroy(temp_interp_data);
+  }
   for(i = 0; i < agent->datastate->tasks[progress]->num_saved_variables; i++) {
+	/*agent_variable_list_Remove(
+        agent->datastate->tasks[progress]->agent_variable_list,
+	agent->datastate->tasks[progress]->saved_variables[i]
+	); */ /* FIXME: Why doesn't this work? */
     agent_variable_list_Add(
         agent->datastate->tasks[progress]->agent_variable_list,
         interpreter_variable_data_Initialize(
@@ -948,6 +1103,9 @@ FIXME: This is silly
         );
   }
 
+  /* Apply progress modifier */
+  agent->datastate->task_progress += agent->datastate->progress_modifier;
+
   if (agent->datastate->persistent || 
       agent->datastate->tasks[progress]->persistent ) {
     /* TODO: We need a large while loop here that waits on a condition 
@@ -964,7 +1122,9 @@ FIXME: This is silly
     ((MCAgent_t) ChAgent)->agent_status = MC_AGENT_NEUTRAL;
   } else {
     if ((((MCAgent_t)ChAgent)->agent_interp) != NULL) {
-      Ch_End(((MCAgent_t)ChAgent)->agent_interp);
+      /* Reset the interpreter and put it back into the queue */
+      Ch_Reset(*agent->agent_interp);
+      interpreter_queue_Add(mc_platform->interpreter_queue, agent->agent_interp);
     }
 
     /* Perform some housekeeping regarding agent status */
@@ -987,7 +1147,7 @@ FIXME: This is silly
       }
     }
     else {
-      ((MCAgent_t) ChAgent)->agent_status = MC_WAIT_MESSGSEND; 
+      ((MCAgent_t) ChAgent)->agent_status = MC_WAIT_MESSGSEND;
     }
   }
 
