@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -127,6 +127,7 @@ agent_datastate_New( void )
   agent_datastate->number_of_tasks = 0;
   agent_datastate->persistent = 0;
   agent_datastate->init_agent_status = 0;
+	agent_datastate->progress_modifier = 0;
 
   return agent_datastate;
 }
@@ -135,6 +136,7 @@ int
 agent_datastate_Destroy( agent_datastate_p agent_datastate )
 {
   int i;
+	if(agent_datastate == NULL) {return 0;}
   if (agent_datastate->agent_codes != NULL) {
     i = 0;
     while (agent_datastate->agent_codes[i] != NULL) {
