@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -65,42 +65,67 @@ agent_task_New(void)
 agent_task_p 
 agent_task_Copy(agent_task_p task)
 {
-  agent_task_p cp_task;
-  cp_task = (agent_task_p)malloc(sizeof(agent_task_t));
+	int i;
+	interpreter_variable_data_t* interp_data;
+	agent_task_p cp_task;
+	cp_task = (agent_task_p)malloc(sizeof(agent_task_t));
 
-  cp_task->number_of_elements = task->number_of_elements;
-  cp_task->size_of_element_array = task->size_of_element_array;
-  cp_task->persistent = task->persistent;
-  cp_task->init_agent_status = task->init_agent_status;
+	cp_task->number_of_elements = task->number_of_elements;
+	cp_task->size_of_element_array = task->size_of_element_array;
+	cp_task->persistent = task->persistent;
+	cp_task->init_agent_status = task->init_agent_status;
 
-  cp_task->var_name = malloc
-    (
-     sizeof(char) * 
-     (strlen(task->var_name) + 1)
-    );
-  strcpy(cp_task->var_name, task->var_name);
+	cp_task->var_name = malloc
+		(
+		 sizeof(char) * 
+		 (strlen(task->var_name) + 1)
+		);
+	strcpy(cp_task->var_name, task->var_name);
 
-  cp_task->server_name = malloc
-    (
-     sizeof(char) * 
-     (strlen(task->server_name) + 1)
-    );
-  strcpy(cp_task->server_name, task->server_name);
+	cp_task->server_name = malloc
+		(
+		 sizeof(char) * 
+		 (strlen(task->server_name) + 1)
+		);
+	strcpy(cp_task->server_name, task->server_name);
 
-  if (task->code_id != NULL) {
-    cp_task->code_id = malloc
-      (
-       sizeof(char) * 
-       (strlen(task->code_id) + 1)
-      );
-    strcpy(cp_task->code_id, task->code_id);
-  } else {
-    cp_task->code_id = NULL;
-  }
+	if (task->code_id != NULL) {
+		cp_task->code_id = malloc
+			(
+			 sizeof(char) * 
+			 (strlen(task->code_id) + 1)
+			);
+		strcpy(cp_task->code_id, task->code_id);
+	} else {
+		cp_task->code_id = NULL;
+	}
 
-  cp_task->agent_return_data = NULL;
+	if(task->agent_return_data != NULL) {
+		cp_task->agent_return_data = interpreter_variable_data_Copy(task->agent_return_data);
+	}
 
-  return cp_task;
+	/* Copy the agent variable list */
+	cp_task->agent_variable_list = agent_variable_list_New();
+	for(i = 0; i < task->agent_variable_list->size; i++) {
+		interp_data = ListSearch(
+				task->agent_variable_list,
+				i );
+		if (interp_data == NULL) { continue; }
+		interp_data = interpreter_variable_data_Copy(interp_data);
+		ListAdd(
+				cp_task->agent_variable_list,
+				interp_data
+				);
+	}
+
+	cp_task->saved_variables = (char**)malloc(sizeof(char*) * (task->num_saved_variables+1));
+	for(i = 0; i < task->num_saved_variables; i++) {
+		cp_task->saved_variables[i] = strdup(task->saved_variables[i]);
+	}
+	cp_task->saved_variables[i] = NULL;
+	cp_task->num_saved_variables = task->num_saved_variables;
+
+	return cp_task;
 }
 
 int
