@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -117,7 +117,7 @@ ams_ManageAgentList(ams_p ams)
 {
   /*variables */
   MCAgent_t current_agent;
-  int index;
+  int index = 0;
   agent_queue_p alist;
   mc_platform_p global;
   message_p message;
@@ -132,15 +132,16 @@ ams_ManageAgentList(ams_p ams)
   {
     if((current_agent = ListSearch(alist->list, index)))
     {
+			if(current_agent->binary) {continue;} /* Do not deal with binary agents */
       MUTEX_UNLOCK(alist->lock);
       MUTEX_LOCK(current_agent->lock);
       current_agent->orphan = 0;
       MUTEX_LOCK(global->quit_lock);
-      if ( global->quit && current_agent->agent_status != MC_WAIT_MESSGSEND) {
+      if(global->quit && current_agent->agent_status != MC_WAIT_MESSGSEND) {
         MUTEX_UNLOCK(global->quit_lock);
         MUTEX_UNLOCK(current_agent->lock);
         MC_TerminateAgent(current_agent);
-        /* If we can lock the agent's run lock, then it must not bu running. */
+        /* If we can lock the agent's run lock, then it must not be running. */
         MUTEX_LOCK(current_agent->run_lock);
         MUTEX_UNLOCK(current_agent->run_lock);
         continue;
@@ -173,13 +174,14 @@ ams_ManageAgentList(ams_p ams)
               )
              )
           {
+	    fprintf(stderr, "Error initializing message from agent. %s:%d\n", __FILE__, __LINE__);
             message_Destroy(message);
             message = NULL;
           } else {
             /* Rename the agent: This is done so that if the agent is coming back here
              * very quickly, the names will not conflict. */
-            current_agent->name = realloc( 
-                current_agent->name, 
+            current_agent->name = realloc(
+                current_agent->name,
                 sizeof(char) * (strlen(current_agent->name) + 10)
                 );
             strcat(current_agent->name, "_SENDING");
@@ -195,12 +197,15 @@ ams_ManageAgentList(ams_p ams)
         case MC_WAIT_FINISHED :
           MUTEX_UNLOCK(current_agent->lock);
           agent_queue_RemoveIndex(alist, index);
-          index=0;
+          // Change index = 0 to index-- to fix the problem where agents 
+          // remain in an agency when they should be removed from the agency.
+          // Yu-Cheng Chou July 28, 2009
+          index--;
           break;
         default : 
           printf("ERROR IN AGENT FORMAT"); 
           printf("Agent Format %d not recognized.",
-              current_agent->agent_status);
+                 current_agent->agent_status);
           /* Flush the invalid agent. */
           current_agent->agent_status = MC_WAIT_FINISHED;
           MUTEX_UNLOCK(current_agent->lock);
@@ -254,7 +259,7 @@ ams_Thread( LPVOID arg )
 {
   mc_platform_p mc_platform = (mc_platform_p)arg;
   ams_p ams = mc_platform->ams;
-
+  int ams_thread_count = 0; 
   while(1) {
     MUTEX_LOCK(ams->runflag_lock);
     MUTEX_LOCK(mc_platform->quit_lock);
@@ -288,6 +293,7 @@ ams_Thread( LPVOID arg )
     MUTEX_UNLOCK(mc_platform->quit_lock);
     MUTEX_UNLOCK(ams->runflag_lock);
     ams_ManageAgentList(ams);
+    ams_thread_count++;
   }
   THREAD_EXIT();
 }
