@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -116,7 +116,7 @@ barrier_queue_Delete(int id, barrier_queue_p list)
   for (i = 0; i < list->list->size; i++) {
     tmp = (barrier_node_p)ListSearch(list->list, i);
     if (tmp->id == id) {
-      ListDelete(list->list, id);
+      ListDelete(list->list, i);
       barrier_node_Destroy(tmp);
       list->size--;
       RWLOCK_WRUNLOCK(list->lock);
