@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -53,6 +53,70 @@
 #include <readline/history.h>
 #endif
 
+/* 
+ * Readline / History Functions
+ */
+#ifdef HAVE_LIBREADLINE
+int initialize_readline (void);
+char ** command_completion (char* text, int start, int end);
+char * command_generator (char* text, int state);
+
+int initialize_readline (void)
+{
+  rl_readline_name = "$";
+  rl_attempted_completion_function = (CPPFunction *)command_completion;
+  return 0;
+}
+
+char ** command_completion (char* text, int start, int end)
+{
+  char **matches;
+  matches = (char **)NULL;
+
+  if (start == 0)
+    matches = (char**)completion_matches (text, command_generator);
+
+  return (matches);
+}
+
+char * command_generator (char* text, int state)
+{
+  static int list_index, len;
+  char *name;
+
+  if (!state)
+  {
+    list_index = 0;
+    len = strlen (text);
+  }
+
+  while (name = command_cmds[list_index])
+  {
+    list_index++;
+
+    if (strncmp (name, text, len) == 0)
+      return (strdup(name));
+  }
+
+  return ((char *)NULL);
+}
+
+// STRIPWHITE ------------------------------------------------------------------
+//   Strip whitespace from the start and end of STRING.
+//   Return a pointer into STRING.
+char* stripwhite (char* string)
+{
+  register char *s, *t;
+  for (s = string; whitespace (*s); s++);
+  if (*s == 0) return (s);
+  t = s + strlen (s) - 1;
+  while (t > s && whitespace (*t)) t--;
+  *++t = '\0';
+  return s;
+}
+// STRIPWHITE ------------------------------------------------------------------
+#endif /* HAVE_LIBREADLINE */
+
 cmd_prompt_p
 cmd_prompt_Initialize(mc_platform_p mc_platform)
 {
@@ -114,11 +178,17 @@ cmd_prompt_Thread( LPVOID arg )
     printf("\n");
 	buf = (char*)malloc(sizeof(char) * 100);
 	if (!buf) {fprintf(stderr, "Malloc failed at %s:%d.\n", __FILE__, __LINE__); }
+#ifdef HAVE_LIBREADLINE
+  initialize_readline();
+#endif
     while(1)
     {
         /* We want to repeatedly grab and execute the user's commands. */
 #ifdef HAVE_LIBREADLINE
         buf = readline("MobileC > ");
+        if (*buf) {
+          add_history(buf);
+        }
 #else
 		printf("MobileC > ");
 #ifndef _WIN32
@@ -139,6 +209,9 @@ cmd_prompt_Thread( LPVOID arg )
         process_command(&cmd);
         exec_command(cmd, mc_platform);
         dealloc_command(&cmd);
+#ifdef HAVE_LIBREADLINE
+        free(buf);
+#endif
     }
     return 0;
 } /*}}}*/
@@ -325,3 +398,9 @@ int handler_PRINTLIST_AGENTS(void *arg, mc_platform_p global) /*{{{*/
     return 0;
 } /*}}}*/
 
+int handler_FLUSH_AGENTS(void *arg, mc_platform_p global) /*{{{*/
+{
+    agent_queue_Flush(global->agent_queue);
+    return 0;
+} /*}}}*/
+
