@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -42,6 +42,7 @@
 #include <string.h>
 #include "include/connection.h"
 #include "include/mc_error.h"
+#include "include/macros.h"
 
 int
 connection_Destroy(connection_p connection)
@@ -51,7 +52,8 @@ connection_Destroy(connection_p connection)
   }
   /* Close the connection, just to make sure. */
 #ifndef _WIN32
-  close(connection->clientfd);
+  /*if(close(connection->clientfd) <0) 
+		SOCKET_ERROR(); */
 #else
   closesocket(connection->clientfd);
 #endif
