@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -184,16 +184,32 @@ agent_queue_Print(agent_queue_p queue)
   for(i = 0; i < queue->size; i++) {
     node = (agent_p)ListSearch(queue->list, i);
     if (node != NULL) {
-      printf("agent id %d:\n\tname:%s\n",
+      printf("agent id %d:\n\tname:%s  status:%d\n",
           (int)node->id,
-          node->name
-          );
+          node->name, node->agent_status);
     }
   }
   MUTEX_UNLOCK(queue->lock);
   return i;
 }
 
+int
+agent_queue_Flush(agent_queue_p queue)
+{
+  int i;
+  agent_p node;
+  MUTEX_LOCK(queue->lock);
+  for(i = 0; i < queue->size; i++) {
+    node = (agent_p)ListSearch(queue->list, i);
+    if (node != NULL) {
+      printf("flushing agent id %d: name:%s status:%d\n",
+          (int)node->id, node->name, node->agent_status);
+      agent_queue_RemoveIndex(queue->list, i);
+    }
+  }
+  MUTEX_UNLOCK(queue->lock);
+  return i;
+}
 /* Mail Queue */
 AP_QUEUE_STD_DEFN_TEMPLATE(
     mail_queue,
@@ -239,3 +255,40 @@ AP_QUEUE_STD_DEFN_TEMPLATE(
     agent_mailbox
     )
 
+/* Interpreter Queue */
+AP_GENERIC_p AP_GENERIC_Copy(AP_GENERIC_p AP_GENERIC);
+
+AP_QUEUE_STD_DEFN_TEMPLATE(
+		interpreter_queue,
+		AP_GENERIC
+		)
+
+AP_GENERIC_p interpreter_queue_CreateRetrieve( struct interpreter_queue_s *queue , ChOptions_t* interp_options) {
+	/* This function retrieves an interpreter from the list if there are any
+		on the queue. Otherwise, it creates a new one and returns it. */
+	struct AP_GENERIC_s *ret;
+	ChInterp_t* interp;
+	MUTEX_LOCK(queue->lock);
+	if (queue->size <= 0) {
+		interp = malloc(sizeof(ChInterp_t));
+		Ch_Initialize(*interp, interp_options);
+    /* Initialize the agent intepreter variables */
+    agent_ChScriptInitVar(interp);
+		ret = interp;
+		MUTEX_UNLOCK(queue->lock);
+		return ret;
+	}
+	ret = ListPop(queue->list);
+	queue->size--;
+	COND_SIGNAL(queue->cond);
+	MUTEX_UNLOCK(queue->lock);
+	return ret;
+}
+
+int AP_GENERIC_Destroy(AP_GENERIC_t* blah) {
+	return 0;
+}
+
+AP_GENERIC_p AP_GENERIC_Copy(AP_GENERIC_p AP_GENERIC) {
+	return NULL;
+}
