@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -73,7 +73,7 @@ int fipa_acl_message_Destroy(fipa_acl_message_t* message)
   fipa_expression_Destroy(message->language);
   fipa_expression_Destroy(message->encoding);
   fipa_expression_Destroy(message->ontology);
-  fipa_word_Destroy(message->protocol);
+  //fipa_word_Destroy(message->protocol);
   fipa_expression_Destroy(message->conversation_id);
   fipa_expression_Destroy(message->reply_with);
   fipa_expression_Destroy(message->in_reply_to);
@@ -96,7 +96,8 @@ fipa_acl_message_t* fipa_acl_message_Copy(fipa_acl_message_t* src)
   copy->language = fipa_expression_Copy(src->language);
   copy->encoding = fipa_expression_Copy(src->encoding);
   copy->ontology = fipa_expression_Copy(src->ontology);
-  copy->protocol = fipa_word_Copy(src->protocol);
+  //copy->protocol = fipa_word_Copy(src->protocol);
+  copy->protocol = src->protocol;
   copy->conversation_id = fipa_expression_Copy(src->conversation_id);
   copy->reply_with = fipa_expression_Copy(src->reply_with);
   copy->in_reply_to = fipa_expression_Copy(src->in_reply_to);
@@ -509,7 +510,8 @@ int fipa_message_parameter_Parse(fipa_acl_message_p acl, fipa_message_string_p m
   } else if (!strcmp(parameter, "ontology")) {
     err = fipa_expression_Parse(&(acl->ontology), message);
   } else if (!strcmp(parameter, "protocol")) {
-    err = fipa_word_Parse(&(acl->protocol), message);
+    //err = fipa_word_Parse(word, message);
+    err = fipa_protocol_type_Parse(&(acl->protocol), message);
   } else if (!strcmp(parameter, "conversation-id")) {
     err = fipa_expression_Parse(&(acl->conversation_id), message);
   } else {
@@ -522,6 +524,46 @@ int fipa_message_parameter_Parse(fipa_acl_message_p acl, fipa_message_string_p m
   return err;
 }
 
+int fipa_protocol_type_Parse(
+    enum fipa_protocol_e* protocol, 
+    fipa_message_string_p message
+    )
+{
+  int err = 0;
+  fipa_word_p word;
+  err = fipa_word_Parse(&word, message);
+  if (err) return err;
+  if(!strcasecmp(word->content, "fipa-request")) {
+    *protocol = FIPA_PROTOCOL_REQUEST;
+  } else if (!strcasecmp(word->content, "fipa-query")) {
+    *protocol = FIPA_PROTOCOL_QUERY;
+  } else if (!strcasecmp(word->content, "fipa-request-when")) {
+    *protocol = FIPA_PROTOCOL_REQUEST_WHEN;
+  } else if (!strcasecmp(word->content, "fipa-contract-net")) {
+    *protocol = FIPA_PROTOCOL_CONTRACT_NET;
+  } else if (!strcasecmp(word->content, "fipa-iterated-contract-net")) {
+    *protocol = FIPA_PROTOCOL_ITERATED_CONTRACT_NET;
+  } else if (!strcasecmp(word->content, "fipa-auction-english")) {
+    *protocol = FIPA_PROTOCOL_ENGLISH_AUCTION;
+  } else if (!strcasecmp(word->content, "fipa-auction-dutch")) {
+    *protocol = FIPA_PROTOCOL_DUTCH_AUCTION;
+  } else if (!strcasecmp(word->content, "fipa-brokering")) {
+    *protocol = FIPA_PROTOCOL_BROKERING;
+  } else if (!strcasecmp(word->content, "fipa-recruiting")) {
+    *protocol = FIPA_PROTOCOL_RECRUITING;
+  } else if (!strcasecmp(word->content, "fipa-subscribe")) {
+    *protocol = FIPA_PROTOCOL_SUBSCRIBE;
+  } else if (!strcasecmp(word->content, "fipa-propose")) {
+    *protocol = FIPA_PROTOCOL_PROPOSE;
+  } else {
+    fprintf(stderr, "Unknown protocol: '%s'. %s:%d\n",
+        word->content, __FILE__, __LINE__);
+    err = MC_ERR_PARSE;
+  }
+  fipa_word_Destroy(word);
+  return err;
+}
+
 int fipa_message_type_Parse(
     enum fipa_performative_e* performative, 
     fipa_message_string_p message
@@ -1227,9 +1269,10 @@ int fipa_acl_Compose(dynstring_t** msg, fipa_acl_message_t* acl)
     fipa_expression_Compose(*msg, acl->ontology);
     dynstring_Append(*msg, "\n");
   }
-  if (acl->protocol) {
+  if (acl->protocol > FIPA_PROTOCOL_NONE) {
     dynstring_Append(*msg, ":protocol ");
-    fipa_word_Compose(*msg, acl->protocol);
+    //fipa_word_Compose(*msg, acl->protocol);
+    fipa_protocol_Compose(*msg, acl->protocol);
     dynstring_Append(*msg, "\n");
   }
   if (acl->conversation_id) {
@@ -1256,6 +1299,49 @@ int fipa_acl_Compose(dynstring_t** msg, fipa_acl_message_t* acl)
   return 0;
 }
 
+int fipa_protocol_Compose(dynstring_t* msg, enum fipa_protocol_e protocol)
+{
+  switch(protocol)
+  {
+    case FIPA_PROTOCOL_REQUEST:                                                                   
+      dynstring_Append(msg, "fipa-request"); 
+      break;
+    case FIPA_PROTOCOL_QUERY:
+      dynstring_Append(msg, "fipa-query"); 
+      break;
+    case FIPA_PROTOCOL_REQUEST_WHEN:                                                              
+      dynstring_Append(msg, "fipa-request-when"); 
+      break;
+    case FIPA_PROTOCOL_CONTRACT_NET:
+      dynstring_Append(msg, "fipa-contract-net"); 
+      break;
+    case FIPA_PROTOCOL_ITERATED_CONTRACT_NET:                                                     
+      dynstring_Append(msg, "fipa-iterated-contract-net"); 
+      break;
+    case FIPA_PROTOCOL_ENGLISH_AUCTION:                                                           
+      dynstring_Append(msg, "fipa-auction-english"); 
+      break;
+    case FIPA_PROTOCOL_DUTCH_AUCTION:                                                             
+      dynstring_Append(msg, "fipa-auction-dutch"); 
+      break;
+    case FIPA_PROTOCOL_BROKERING:
+      dynstring_Append(msg, "fipa-brokering"); 
+      break;
+    case FIPA_PROTOCOL_RECRUITING:                                                                
+      dynstring_Append(msg, "fipa-recruiting"); 
+      break;
+    case FIPA_PROTOCOL_SUBSCRIBE:                                                                 
+      dynstring_Append(msg, "fipa-subscribe"); 
+      break;
+    case FIPA_PROTOCOL_PROPOSE:                                                                   
+      dynstring_Append(msg, "fipa-propose"); 
+      break;
+    default:
+      return MC_ERR_PARSE;
+  }
+  return 0;
+}
+
 int fipa_performative_Compose(dynstring_t* msg, enum fipa_performative_e performative)
 {
   switch(performative) {
@@ -1482,11 +1568,13 @@ struct fipa_acl_message_s* fipa_Reply(
     acl_reply->receiver = fipa_agent_identifier_set_New();
     acl_reply->receiver->num = 1;
     acl_reply->receiver->retain_order = 0;
-
-    acl_reply->receiver->fipa_agent_identifiers = (fipa_agent_identifier_t**)malloc(
-        sizeof(fipa_agent_identifier_t*));
-    acl_reply->receiver->fipa_agent_identifiers[0] = fipa_agent_identifier_Copy(
-        acl->sender );
+    acl_reply->receiver->fipa_agent_identifiers = 
+       (fipa_agent_identifier_t**)malloc(sizeof(fipa_agent_identifier_t*));
+    acl_reply->receiver->fipa_agent_identifiers[0] = 
+       fipa_agent_identifier_Copy(acl->sender );
+    acl_reply->protocol = acl->protocol;
+    if(acl->conversation_id)
+       acl_reply->conversation_id = fipa_expression_Copy(acl->conversation_id);
   }
   
   return acl_reply;
