@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -1054,14 +1054,17 @@ fipa_envelope_Compose__from(fipa_acl_message_t* fipa_acl)
         url_node = mxmlNewElement(
             addresses_node,
             "url" );
-        /*mxmlNewText(
-            url_node,
-            0,
-            fipa_acl->sender->addresses->urls[i]->str );*/
+        // uncommented the following
         mxmlNewText(
             url_node,
             0,
-            "http://localhost:5555/acc");
+            fipa_acl->sender->addresses->urls[i]->str );
+
+        // Why was this hardcoded before?
+        /* mxmlNewText(
+            url_node,
+            0,
+            "http://localhost:5555/acc"); */ 
       }
     }
   } else {
