@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -44,18 +44,32 @@
 
 struct mc_platform_s;
 
-
+struct connection_thread_arg_s;
 STRUCT(
     acc,
     struct mc_platform_s* mc_platform;
     THREAD_T thread;
     THREAD_T message_handler_thread;
     THREAD_T listen_thread;
+		THREAD_T udplisten_thread;
 
     /* waiting flag: Note that this flag only applies to the listen_thread */
     int waiting;
     MUTEX_T* waiting_lock;
     COND_T* waiting_cond;
+
+		/* Connection Queue Threads */
+		int num_conn_threads;
+		THREAD_T conn_thread;
+		MUTEX_T conn_thread_lock;
+		COND_T conn_thread_cond;
+		struct connection_thread_arg_s* connection_thread_arg;
+
+		/* Msg Queue Threads */
+		int num_msg_threads;
+		THREAD_T msg_thread;
+		MUTEX_T msg_thread_lock;
+		COND_T msg_thread_cond;
     )
 
 STRUCT(
@@ -66,6 +80,14 @@ STRUCT(
     unsigned long int server_fd;
     );
 
+struct connection_s;
+STRUCT(
+		connection_thread_arg,
+		struct mc_platform_s* mc_platform;
+		struct connection_s* connection;
+		)
+
+
 acc_p 
 acc_Initialize(struct mc_platform_s* mc_platform);
 
@@ -82,8 +104,14 @@ acc_MessageHandlerThread(void* arg);
 void* 
 acc_Thread(void* arg);
 
+void* 
+acc_connection_Thread(void* arg);
+
 void* 
 listen_Thread(void* arg);
+
+void* 
+udplisten_Thread(void* arg);
 #else
 DWORD WINAPI 
 acc_MessageHandlerThread( LPVOID arg );
@@ -91,8 +119,14 @@ acc_MessageHandlerThread( LPVOID arg );
 DWORD WINAPI 
 acc_Thread( LPVOID arg );
 
+DWORD WINAPI 
+acc_connection_Thread( LPVOID arg );
+
 DWORD WINAPI 
 listen_Thread( LPVOID arg );
+
+DWORD WINAPI 
+udplisten_Thread( LPVOID arg );
 #endif /* _WIN32 */
 
 #endif 
