@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -61,6 +61,9 @@ struct agent_s {
   char *home;
   char *sender;
   int home_port;
+	char *wg_code; /* The workgroup code of the agent. Only agents of the same workgroup should
+                    be able to manipulate each other. */
+	char *agent_address;
 
   int orphan; /* If an agent is an 'orphan', it means it is not attached to an
                  agency. */
@@ -73,7 +76,7 @@ struct agent_s {
   int return_data;
 
   /* threading variables */
-  ChInterp_t agent_interp;
+  ChInterp_t* agent_interp;
   /* run_lock should be locked whenever the interpreter is running
    * to prevent simultaneous runs. */
   MUTEX_T *run_lock;
@@ -98,6 +101,8 @@ struct agent_s {
   struct mc_platform_s *mc_platform;
 
   MUTEX_T* lock;
+
+	int binary; /* Indicates whether or not this is a binary space agent */
 };
 #ifndef AGENT_T
 #define AGENT_T
@@ -112,6 +117,9 @@ agent_AddPersistentVariable(agent_p agent, int task_num, const char* var_name);
 agent_p
 agent_New(void);
 
+agent_p 
+agent_NewBinary( struct mc_platform_s *mc_platform);
+
 agent_p
 agent_Copy(const agent_p agent);
 
@@ -134,4 +142,7 @@ void * agent_RunChScriptThread(void * agent);
 DWORD WINAPI agent_RunChScriptThread(void * ChAgent);
 #endif
 
+void*
+agent_ChScriptInitVar(ChInterp_t* interp);
+
 #endif 
