@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -57,6 +57,12 @@ MC_AclWaitRetrieve_chdl(void *varg);
 
 /* BEGIN Acl helper functions */
 
+EXPORTCH void*
+MC_AclSetProtocol_chdl(void *varg);
+
+EXPORTCH void*
+MC_AclSetConversationID_chdl(void *varg);
+
 EXPORTCH void*
 MC_AclSetPerformative_chdl(void *varg);
 
@@ -112,6 +118,9 @@ MC_CondSignal_chdl(void *varg);
 EXPORTCH int
 MC_DeleteAgent_chdl(void *varg);
 
+EXPORTCH int
+MC_DeleteAgentWG_chdl(void *varg);
+
 EXPORTCH int
 MC_DeregisterService_chdl(void *varg); 
 
@@ -196,6 +205,9 @@ MC_SendSteerCommand_chdl(void *varg);
 EXPORTCH int
 MC_TerminateAgent_chdl(void *varg);
 
+EXPORTCH int
+MC_TerminateAgentWG_chdl(void *varg);
+
 EXPORTCH int
 MC_GetAgentStatus_chdl(void *varg);
 
