@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -36,6 +36,12 @@
 #define _AP_QUEUE_TEMPLATE_H_
 #include "macros.h"
 #include "mc_error.h"
+#include <embedch.h>
+
+struct AP_GENERIC_s {void* none;};
+typedef struct AP_GENERIC_s AP_GENERIC_t;
+typedef AP_GENERIC_t* AP_GENERIC_p;
+
 /* Fields will be: NAME */
 #define AP_QUEUE_DECL_TEMPLATE( name, node_type) \
 typedef struct name##_s \
@@ -53,6 +59,7 @@ int name##_Destroy( name##_p name ); \
 int name##_Add( name##_p name, struct node_type##_s* node ); \
 name##_p name##_Copy(name##_p name); \
 struct node_type##_s* name##_Pop( name##_p name ); \
+struct node_type##_s* name##_WaitPop( name##_p name ); \
 struct node_type##_s* name##_SearchIndex( name##_p name, int index ); \
 int name##_RemoveIndex(name##_p name, int index); 
 
@@ -149,6 +156,20 @@ node_type##_t* name##_Pop( name##_p name ) \
   return ret; \
 } \
   \
+node_type##_t* name##_WaitPop( name##_p name ) \
+{ \
+  struct node_type##_s* ret; \
+  MUTEX_LOCK(name->lock); \
+  while (name->size <= 0) { \
+		COND_WAIT(name->cond, name->lock); \
+  } \
+  ret = ListPop(name->list); \
+  name->size--; \
+  COND_SIGNAL(name->cond); \
+  MUTEX_UNLOCK(name->lock); \
+  return ret; \
+} \
+  \
 struct node_type##_s* name##_SearchIndex( name##_p name, int index ) \
 { \
   struct node_type##_s* node; \
@@ -228,4 +249,8 @@ int name##_##func_name( name##_p name, const search_type key ) \
   MUTEX_UNLOCK(name->lock); \
   return err_code; \
 }
+
+/* Custom function for interpreter queues */
+struct interpreter_queue_s;
+AP_GENERIC_p interpreter_queue_CreateRetrieve( struct interpreter_queue_s *queue, ChOptions_t* interp_options );
 #endif
