@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -55,4 +55,7 @@ COMMAND(PRINTLIST_AGENTS,
         "printlist_agents",
         "This command prints all the agents on the system which are still\n\
         on the agent list.\n")
-
+COMMAND(FLUSH_AGENTS,
+        "flush_agents",
+        "This command flushes all of the agents on the system which are still\n\
+        on the agent list.\n")
