@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -178,4 +178,11 @@ AP_QUEUE_GENERIC_DECL_TEMPLATE(
     int
     )
 
+/* Interpereter Queue */
+AP_QUEUE_DECL_TEMPLATE(
+		interpreter_queue,
+		AP_GENERIC
+		)
+
+
 #endif
