@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -67,7 +67,29 @@ enum fipa_performative_e
 };
 #endif
 
+#ifndef _FIPA_PROTOCOL_E_
+#define _FIPA_PROTOCOL_E_
+enum fipa_protocol_e
+{
+  FIPA_PROTOCOL_ERROR=-1,
+  FIPA_PROTOCOL_NONE,       // default = 0
+  FIPA_PROTOCOL_REQUEST,                                                                   
+  FIPA_PROTOCOL_QUERY,
+  FIPA_PROTOCOL_REQUEST_WHEN,                                                              
+  FIPA_PROTOCOL_CONTRACT_NET,
+  FIPA_PROTOCOL_ITERATED_CONTRACT_NET,                                                     
+  FIPA_PROTOCOL_ENGLISH_AUCTION,                                                           
+  FIPA_PROTOCOL_DUTCH_AUCTION,                                                             
+  FIPA_PROTOCOL_BROKERING,
+  FIPA_PROTOCOL_RECRUITING,                                                                
+  FIPA_PROTOCOL_SUBSCRIBE,                                                                 
+  FIPA_PROTOCOL_PROPOSE,                                                                   
+  FIPA_PROTOCOL_END                   
+};
+#endif
+
 int fipa_performative_Compose(dynstring_t* msg, enum fipa_performative_e performative);
+int fipa_protocol_Compose(dynstring_t* msg, enum fipa_protocol_e protocol);
 
 typedef enum fipa_expression_type_e
 {
@@ -100,7 +122,8 @@ typedef struct fipa_acl_message_s
   
   struct fipa_expression_s* ontology;
 
-  struct fipa_word_s* protocol;
+  //struct fipa_word_s* protocol;
+  enum fipa_protocol_e protocol;
 
   struct fipa_expression_s* conversation_id;
 
@@ -252,14 +275,9 @@ int fipa_number_Compose(dynstring_t* msg, fipa_number_t* number);
 /* Parsing Functions */
 int fipa_acl_Parse(struct fipa_acl_message_s* acl, fipa_message_string_p message);
 int fipa_message_parameter_Parse(struct fipa_acl_message_s* acl, fipa_message_string_p message);
-int fipa_message_type_Parse(
-    enum fipa_performative_e* performative, 
-    fipa_message_string_p message
-    );
-int fipa_GetAtom(
-    fipa_message_string_p message,
-    char expected_atom
-    );
+int fipa_protocol_type_Parse(enum fipa_protocol_e* protocol, fipa_message_string_p message);
+int fipa_message_type_Parse(enum fipa_performative_e* performative, fipa_message_string_p message);
+int fipa_GetAtom( fipa_message_string_p message, char expected_atom);
 int fipa_word_Parse(fipa_word_t** word, fipa_message_string_p message);
 int fipa_word_Destroy(fipa_word_t* word);
 int fipa_CheckNextToken(const fipa_message_string_p message, const char* token);
