@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -125,6 +125,15 @@ MC_WaitSignal
 #ifndef _MOBILEC_H_
 #define _MOBILEC_H_
 
+#ifndef THREAD_T
+#ifdef _WIN32
+#include <windows.h>
+#define THREAD_T HANDLE
+#else
+#define THREAD_T pthread_t
+#endif
+#endif
+
 #ifdef WIN32
 	#ifdef _MC_DLL
 		// Building a .dll - export functions as marked
@@ -219,6 +228,7 @@ typedef struct agency_s {
     char *filename;
     int portno;                         /*!< Local port number */
     int portnoc;
+    int initInterps;
     struct mc_platform_s* mc_platform;  /*!< Local MobileC Platform */
     int default_agentstatus;            /*!< Agency default agent status */
     int threads;                        /*!< flag which determines which threads to start */
@@ -245,10 +255,19 @@ typedef struct MCAgencyOptions_s{
     /* Following are some thread stack size options: unix/pthreads only! */
     int stack_size[MC_THREAD_ALL]; /*!< If the stack size is -1, use system def.*/
 
-		char *known_host_filename;
-		char *priv_key_filename;
+    char *known_host_filename;
+    char *priv_key_filename;
+    int initInterps;
+    ChOptions_t* ch_options;
 } MCAgencyOptions_t;
 
+typedef struct agent_thread_arg_s {
+	void* args;
+	struct agent_s* agent;
+	MCAgency_t attr;
+	THREAD_T thread;
+} agent_thread_arg_t;
+
 #ifndef AGENT_T
 #define AGENT_T
 typedef struct agent_s agent_t;
@@ -375,6 +394,15 @@ EXPORTMC extern struct fipa_acl_message_s* MC_AclWaitRetrieve(MCAgent_t agent);
 
 /* ACL Helper Functions Here */
 enum fipa_performative_e;
+enum fipa_protocol_e;
+
+EXPORTMC int MC_AclSetProtocol(
+    struct fipa_acl_message_s* acl,
+    enum fipa_protocol_e performative );
+
+EXPORTMC int MC_AclSetConversationID(
+    struct fipa_acl_message_s* acl,
+    char* id);
 
 EXPORTMC int MC_AclSetPerformative(
     struct fipa_acl_message_s* acl,
@@ -414,6 +442,13 @@ EXPORTMC extern int MC_AddAgent(
     MCAgency_t attr, 
     MCAgent_t agent);
 
+
+#ifndef _WIN32
+int MC_AddStationaryAgent(MCAgency_t agency, void* (*agent_thread)(struct agent_thread_arg_s*), const char* name, void* agent_args);
+#else
+int MC_AddStationaryAgent(MCAgency_t agency, LPTHREAD_START_ROUTINE agent_thread, const char* name, void* agent_args);
+#endif
+
 /**
  * \brief           Retrieve a pointer to a previously saved variable.
  *
@@ -515,7 +550,9 @@ EXPORTMC extern int MC_BarrierInit(MCAgency_t attr, int id, int num_procs);
  *
  * \return          0 on success, error_code_t type on failure
  */
+#ifdef OBS
 EXPORTMC extern int MC_ChInitializeOptions(MCAgency_t attr, ChOptions_t *options);
+#endif
 
 
 /**
@@ -606,11 +643,12 @@ EXPORTMC extern int MC_CallAgentFuncVar
  * \param code      The agent code
  * \param return_var_name The name of the agent's return variable. Set to "no-return" if no return variable is desired.
  * \param server    The target destination of the agent.
+ * \param workgroup_code The secret workgroup code of the agent. Only agents with the same workgroup code may perform certain interactions.
  * \param persitent A flag indicating whether or not the agent should be persistent. A value of '1' indicates persistence, while a value of '0' indicates default non-persistent behaviour.
  *
  * \return          This function returns a valid MCAgent_t structure on success or NULL on failure.
  */
-MCAgent_t
+EXPORTMC extern MCAgent_t
 MC_ComposeAgent(
     const char* name,
     const char* home,
@@ -618,6 +656,34 @@ MC_ComposeAgent(
     const char* code,
     const char* return_var_name,
     const char* server,
+		const char* workgroup_code,
+    int persistent
+    );
+
+/**
+ * \brief           Compose a new agent dynamically from a source code file
+ *
+ * \param filename  The filename of the source file
+ * \param name      The desired name of the new agent.
+ * \param home      The home of the new agent.
+ * \param owner     The owner of the new agent.
+ * \param code      The agent code
+ * \param return_var_name The name of the agent's return variable. Set to "no-return" if no return variable is desired.
+ * \param server    The target destination of the agent.
+ * \param workgroup_code The secret workgroup code of the agent. Only agents with the same workgroup code may perform certain interactions.
+ * \param persitent A flag indicating whether or not the agent should be persistent. A value of '1' indicates persistence, while a value of '0' indicates default non-persistent behaviour.
+ *
+ * \return          This function returns a valid MCAgent_t structure on success or NULL on failure.
+ */
+EXPORTMC extern MCAgent_t
+MC_ComposeAgentFromFile(
+    const char* name,
+    const char* home,
+    const char* owner, 
+    const char* filename,
+    const char* return_var_name,
+    const char* server,
+		const char* workgroup_code,
     int persistent
     );
 
@@ -698,6 +764,19 @@ int MC_CopyAgent(MCAgent_t* agent_out, const MCAgent_t agent_in);
  */
 EXPORTMC extern int MC_DeleteAgent(MCAgent_t agent);
 
+/**
+ * \brief           Stop and remove an agent in the same workgroup
+ *
+ * \param calling_agent The calling agent
+ * \param agent     An agent in any state (running, waiting, etc)
+ *
+ * \note            The agents must belong to the same workgroup.
+ *
+ * \return          0 on success, error_code_t type on failure
+ *
+ */
+EXPORTMC extern int MC_DeleteAgentWG(MCAgent_t calling_agent, MCAgent_t agent);
+
 /**
  * \brief           End an agency
  *
@@ -1195,6 +1274,9 @@ EXPORTMC extern int MC_SyncInit(MCAgency_t attr, int id);
  */
 EXPORTMC extern int MC_TerminateAgent(MCAgent_t agent);
 
+
+EXPORTMC extern int 
+MC_TerminateAgentWG(MCAgent_t calling_agent, MCAgent_t agent);
 /**
  * \brief           Wait indefinitely
  *
