@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -87,6 +87,7 @@
 #include <pthread.h>
 #include <semaphore.h>
 #include "config.h"
+#include <errno.h>
 #else
 #include <windows.h>
 #endif
@@ -128,18 +129,29 @@ typedef name##_t* name##_p;
  * * * * * * * * * * * * */               /* Windows macros follow */
 #ifndef _WIN32
 
+#define SOCKET_ERROR() \
+	printf("Socket error. %s:%d\nerrno:%d", __FILE__, __LINE__, errno); \
+	sleep(500)
+
 /* ****** *
  * THREAD *
  * ****** */
 #define PTHREAD_STACK_SIZE 131072 /* This is PTHREAD_STACK_MIN * 8 for me */
+
+#ifndef THREAD_T
 #define THREAD_T pthread_t
+#endif
+
 #define THREAD_CREATE( thread_handle, function, arg ) \
-  pthread_create( \
+  while(pthread_create( \
       thread_handle, \
       &attr, \
       function, \
       (void*) arg \
-      )
+      ) < 0) { \
+			printf("pthread_create failed. Trying again...\n"); \
+			usleep(100000); \
+	}
 
 #define THREAD_CANCEL( thread_handle ) \
   pthread_cancel( thread_handle )
@@ -147,6 +159,11 @@ typedef name##_t* name##_p;
 #define THREAD_JOIN(thread_handle ) \
   pthread_join( thread_handle, NULL )
 
+#define THREAD_DETACH(thread_handle) \
+	if(pthread_detach(thread_handle) < 0) { \
+		printf("pthread_detach failed. %s:%d\n", __FILE__, __LINE__); \
+	}
+
 #define THREAD_EXIT() \
   pthread_exit(NULL)
 
@@ -302,10 +319,14 @@ pthread_cond_wait( queue->touched_signal, queue->thread_mutex )
  * W I N D O W S   M A C R O S *
  * * * * * * * * * * * * * * * */
 
+#define SOCKET_ERROR() 
 /* ******* *
  * THREADS *
  * ******* */
+#ifndef THREAD_T
 #define THREAD_T HANDLE
+#endif
+
 #define THREAD_CREATE(thread_handle, function, arg) \
   *(thread_handle) = CreateThread( \
       NULL, \
@@ -325,6 +346,10 @@ pthread_cond_wait( queue->touched_signal, queue->thread_mutex )
 #define THREAD_EXIT() \
   ExitThread(0)
 
+#define THREAD_DETACH() 
+
+
+
 /* ***** */
 /* MUTEX */
 /* ***** */
