@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -57,6 +57,7 @@ struct mc_platform_s{
 
   char* hostname;
   unsigned short port;
+  int initInterps;
 
   /* These are the standard agency data structs. */
   message_queue_p asm_message_queue;
@@ -104,7 +105,7 @@ struct mc_platform_s{
   int giant;
   MUTEX_T *giant_lock;
   COND_T  *giant_cond;
-  
+
   int quit;
   MUTEX_T *quit_lock;
   COND_T *quit_cond;
@@ -112,12 +113,15 @@ struct mc_platform_s{
   char private_key[1210]; // private key of agency
 
   MCAgency_t agency; /* A pointer to the platform's agency structure */
+
+	/* Need a linked list of Ch interpreters. */
+	interpreter_queue_p interpreter_queue;
 };
 typedef struct mc_platform_s mc_platform_t;
 typedef mc_platform_t* mc_platform_p;
 
 mc_platform_p 
-mc_platform_Initialize(MCAgency_t agency);
+mc_platform_Initialize(MCAgency_t agency, ChOptions_t* ch_options);
 
 int 
 mc_platform_Destroy(mc_platform_p mc_platform);
