@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -109,9 +109,17 @@ typedef struct message_s{
   /* This flag is set if an agent is using the xml nodes, so we know
    * not to free/delete them prematurely. */
   int agent_xml_flag;
+	char* sending_agent_name;
 } message_t;
 typedef message_t* message_p;
 
+typedef struct message_send_arg_s
+{
+	struct mc_platform_s* mc_platform;
+	message_p message;
+	char* privatekey;
+} message_send_arg_t;
+
 #ifdef MC_SECURITY
 int
 message_Decrypt(message_p message, struct asm_node_s* asm_node);
@@ -159,6 +167,14 @@ auth_rece_send_msg(int sockfd, char *hostname, char *message, char *privkey, cha
 int 
 message_Send(struct mc_platform_s* mc_platform, message_p message, char *privatekey);
 
+#ifndef _WIN32
+void* 
+message_send_Thread(void* arg);
+#else
+DWORD WINAPI
+message_send_Thread(LPVOID arg);
+#endif
+
 int 
 http_to_hostport(const char* http_str, char** host, int* port, char** target);
 #endif
