@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -121,6 +121,8 @@ MC_AclSend(MCAgency_t attr, fipa_acl_message_t* acl)
   MCAgent_t agent;
   int num_addresses = 0;
 
+	fipa_acl_message_t* tmp;
+
   err = fipa_acl_Compose(&msg_string, acl);
   if( err ) {
     fprintf(stderr, "ACL Message Compose Error. %s:%d\n", __FILE__, __LINE__);
@@ -135,6 +137,8 @@ MC_AclSend(MCAgency_t attr, fipa_acl_message_t* acl)
     } else {
       num_addresses = acl->receiver->fipa_agent_identifiers[i]->addresses->num;
     }
+
+
     if (num_addresses == 0) {
       agent = MC_FindAgentByName(
           attr,
@@ -143,8 +147,10 @@ MC_AclSend(MCAgency_t attr, fipa_acl_message_t* acl)
         fprintf(stderr, "Could not find local agent:%s. %s:%d\n",
             acl->receiver->fipa_agent_identifiers[i]->name,
             __FILE__, __LINE__);
+				continue;
       }
-      MC_AclPost(agent, acl);
+			tmp = fipa_acl_message_Copy(acl);
+      MC_AclPost(agent, tmp);
     } else {
       msg = mtp_http_New();
       /* Send to the first address listed */
@@ -185,7 +191,8 @@ MC_AclSend(MCAgency_t attr, fipa_acl_message_t* acl)
          mc_message,
 				 attr->mc_platform->private_key
         );
-      message_Destroy(mc_message);
+      //message_Destroy(mc_message);
+			// We may no longer destroy the message here since we do not know how long it will take message_Send to complete.
       mtp_http_Destroy(msg);
       free(host);
       free(target);
@@ -203,6 +210,41 @@ MC_AclWaitRetrieve(MCAgent_t agent)
 
 /* ACL Helper Functions Here */
 
+int MC_AclSetProtocol(
+    fipa_acl_message_t* acl,
+    enum fipa_protocol_e protocol)
+{
+  if(protocol <= FIPA_PROTOCOL_ERROR ||
+     protocol >= FIPA_PROTOCOL_END)
+  {
+    fprintf(stderr, 
+      "AclSetProtocol: Incorrect protocol. %s:%d\n", 
+      __FILE__, __LINE__);
+    return -1;
+  }
+  acl->protocol = protocol;
+  return 0;
+}
+
+int MC_AclSetConversationID(
+    fipa_acl_message_t* acl,
+    char* id)
+{
+
+  if(!id)
+  {
+    fprintf(stderr, 
+      "AclSetConversationID: NULL id string. %s:%d\n", 
+      __FILE__, __LINE__);
+    return -1;
+  }
+  acl->conversation_id = fipa_expression_New();
+  acl->conversation_id->type = FIPA_EXPR_STRING;
+  acl->conversation_id->content.string = fipa_string_New();
+  acl->conversation_id->content.string->content = strdup(id);
+  return 0;
+}
+
 int MC_AclSetPerformative(
     fipa_acl_message_t* acl,
     enum fipa_performative_e performative )
@@ -335,11 +377,33 @@ MC_AddAgent(MCAgency_t attr, MCAgent_t agent) /*{{{*/
   return 0;
 } /*}}}*/
 
+#ifndef _WIN32
+int MC_AddStationaryAgent(MCAgency_t agency, void* (*agent_thread)(struct agent_thread_arg_s*), const char* name, void* agent_args)
+#else
+int MC_AddStationaryAgent(MCAgency_t agency, LPTHREAD_START_ROUTINE agent_thread, const char* name, void* agent_args)
+#endif
+{
+#ifndef _WIN32
+	pthread_attr_t attr;
+	pthread_attr_init(&attr);
+#else
+	int stack_size = 0;
+#endif
+	struct agent_thread_arg_s* agent_thread_arg;
+	agent_thread_arg = (agent_thread_arg_t*)malloc(sizeof(agent_thread_arg_t));
+	agent_thread_arg->args = agent_args;
+	agent_thread_arg->agent = agent_NewBinary(agency->mc_platform);
+	agent_thread_arg->agent->name = strdup(name);
+	agent_thread_arg->attr = agency;
+	agent_queue_Add(agency->mc_platform->agent_queue, agent_thread_arg->agent);
+	THREAD_CREATE(&agent_thread_arg->thread, agent_thread, agent_thread_arg);
+}
+
 const void* MC_AgentVariableRetrieve(MCAgent_t agent, const char* var_name, int task_num)
 {
   interpreter_variable_data_t* interp_var;
 
-  if (task_num >= agent->datastate->task_progress) {
+  if (task_num > agent->datastate->task_progress) {
     return NULL;
   }
 
@@ -453,7 +517,7 @@ MC_CallAgentFunc(
   va_start(vl, numArgs);
   MUTEX_LOCK(agent->run_lock);
   return_code = Ch_CallFuncByNamev(
-      agent->agent_interp,
+      *agent->agent_interp,
       funcName,
       returnVal,
       vl );
@@ -511,7 +575,7 @@ MC_CallAgentFuncVar(
   MUTEX_LOCK(agent->run_lock);
   return_code = Ch_CallFuncByNameVar
     (
-     agent->agent_interp,
+     *agent->agent_interp,
      funcName,
      returnVal,
      varg
@@ -520,6 +584,7 @@ MC_CallAgentFuncVar(
   return return_code;
 }
 
+#ifdef OBS
 EXPORTMC int 
 MC_ChInitializeOptions(MCAgency_t attr, ChOptions_t *options) { /*{{{*/
     if(attr->mc_platform == NULL) {
@@ -537,8 +602,9 @@ MC_ChInitializeOptions(MCAgency_t attr, ChOptions_t *options) { /*{{{*/
         return 0;
     }
 } /*}}}*/
+#endif
 
-MCAgent_t
+EXPORTMC MCAgent_t
 MC_ComposeAgent(
     const char* name,
     const char* home,
@@ -546,6 +612,7 @@ MC_ComposeAgent(
     const char* code,
     const char* return_var_name,
     const char* server,
+		const char* workgroup_code,
     int persistent
     )
 {
@@ -604,10 +671,55 @@ MC_ComposeAgent(
   if(agent->datastate->tasks[0]->server_name == NULL) {
     fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
   }
+
+	if(workgroup_code != NULL) {
+		agent->wg_code = strdup(workgroup_code);
+		if (agent->wg_code == NULL) {
+			fprintf(stderr, "Memory Error %s:%d\n", __FILE__, __LINE__);
+		}
+	}
   
   return agent;
 }
 
+EXPORTMC MCAgent_t MC_ComposeAgentFromFile(
+    const char* name,
+    const char* home,
+    const char* owner, 
+    const char* filename,
+    const char* return_var_name,
+    const char* server,
+		const char* workgroup_code,
+    int persistent
+    )
+{
+  MCAgent_t agent;
+  struct stat buffer;
+  char *code;
+  int codesize;
+  FILE* fptr;
+
+  stat(filename, &buffer);
+  codesize = buffer.st_size;
+  code = (char *)malloc((codesize+1)*sizeof(char));
+  memset(code, 0, codesize+1);
+  fptr = fopen(filename, "r");
+  fread(code, sizeof(char), codesize, fptr);
+  fclose(fptr);
+
+  agent = MC_ComposeAgent(
+     name, 
+     home, 
+     owner, 
+     code, 
+     return_var_name, 
+     server,
+		 workgroup_code,
+     persistent);
+
+  return agent;
+}
+
 EXPORTMC int 
 MC_CondBroadcast(MCAgency_t attr, int id) /*{{{*/
 {
@@ -691,6 +803,11 @@ MC_DeleteAgent(MCAgent_t agent) /*{{{*/
     /* Error Checking */
     CHECK_NULL(agent, return MC_ERR_INVALID;);
 
+    /* If the agent has a workgroup code, when the user must use MC_DeleteAgentWG() */
+    if ( agent->wg_code != NULL ) {
+      return MC_ERR_INVALID_ARGS;
+    }
+
     /* First, make sure the agent is no longer running */
     MC_TerminateAgent(agent);
 
@@ -700,6 +817,30 @@ MC_DeleteAgent(MCAgent_t agent) /*{{{*/
     return MC_SUCCESS;
 } /*}}}*/
 
+EXPORTMC int 
+MC_DeleteAgentWG(MCAgent_t calling_agent, MCAgent_t agent) /*{{{*/
+{
+    /* Error Checking */
+    CHECK_NULL(agent, return MC_ERR_INVALID;);
+
+    /* The agents must belong to the same workgroup*/
+    if (agent->wg_code != NULL) {
+      if (calling_agent->wg_code == NULL) {
+        return MC_ERR_INVALID_ARGS;
+      } else if (strcmp(calling_agent->wg_code, agent->wg_code)) {
+        return MC_ERR_INVALID_ARGS;
+      }
+    }
+
+    /* First, make sure the agent is no longer running */
+    MC_TerminateAgentWG(calling_agent, agent);
+
+    /* Now, we just set it's status to MC_WAIT_FINISHED so that the AMS will
+     * flush it next chance it gets. */
+    MC_SetAgentStatus(agent, MC_WAIT_FINISHED);
+    return MC_SUCCESS;
+} /*}}}*/
+
 int MC_DestroyServiceSearchResult(
     char** agentName,
     char** serviceName,
@@ -1075,9 +1216,11 @@ MC_Initialize( /*{{{*/
   for(i = 0; i < MC_THREAD_ALL; i++) {
     ret->stack_size[i] = options->stack_size[i];
   }
+  ret->initInterps = options->initInterps;
   /* End agency options */
 
-  ret->mc_platform = mc_platform_Initialize(ret);
+  ret->mc_platform = mc_platform_Initialize(ret, options->ch_options);
+  //ret->mc_platform->interp_options = options->ch_options;
 
 #ifdef NEW_SECURITY
 	if (options->priv_key_filename != NULL) {
@@ -1147,6 +1290,7 @@ EXPORTMC int
 MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options) /*{{{*/
 {
   int i;
+  memset(options, 0, sizeof(struct MCAgencyOptions_s));
   /* Set the default options */
   options->threads = 0xFFFF;
   options->default_agent_status = MC_WAIT_CH;
@@ -1158,6 +1302,8 @@ MC_InitializeAgencyOptions(struct MCAgencyOptions_s* options) /*{{{*/
 
 	options->known_host_filename = NULL;
 	options->priv_key_filename = NULL;
+
+  options->initInterps = 4; // 4 interpreters are loaded
   return 0;
 } /*}}}*/
 
@@ -1240,7 +1386,7 @@ MC_MigrateAgent(MCAgent_t agent, const char* hostname, int port)
   progress = agent->datastate->task_progress;
   
   /* Rewind the task progress */
-  agent->datastate->task_progress--;
+  agent->datastate->progress_modifier = -1;
 
   /* Set the new servername */
   free(agent->datastate->tasks[progress]->server_name);
@@ -1783,11 +1929,28 @@ MC_TerminateAgent(MCAgent_t agent) /*{{{*/
 {
   int status=0;
   if(agent->agent_interp != NULL) {
-    status = Ch_Abort (agent->agent_interp);
+    status = Ch_Abort (*agent->agent_interp);
   }
   return status;
 } /*}}}*/
 
+EXPORTMC int 
+MC_TerminateAgentWG(MCAgent_t calling_agent, MCAgent_t agent) /*{{{*/
+{
+  int status=0;
+  /* The agents must belong to the same workgroup*/
+  if (agent->wg_code != NULL) {
+    if (calling_agent->wg_code == NULL) {
+      return MC_ERR_INVALID_ARGS;
+    } else if (strcmp(calling_agent->wg_code, agent->wg_code)) {
+      return MC_ERR_INVALID_ARGS;
+    }
+  }
+  if(agent->agent_interp != NULL) {
+    status = Ch_Abort (*agent->agent_interp);
+  }
+  return status;
+} /*}}}*/
 
 #ifdef _WIN32
 EXPORTMC BOOL
@@ -1983,6 +2146,43 @@ MC_AclWaitRetrieve_chdl(void *varg)
 
 /* BEGIN Acl helper functions */
 
+/* MC_AclSetProtocol */
+EXPORTCH int 
+MC_AclSetProtocol_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  enum fipa_protocol_e protocol;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  protocol = Ch_VaArg(interp, ap, enum fipa_protocol_e);
+  retval = MC_AclSetProtocol(acl, protocol);
+  Ch_VaEnd(interp, ap);
+  return retval;
+
+}
+
+/* MC_AclSetContentID */
+EXPORTCH int
+MC_AclSetConversationID_chdl(void* varg)
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  struct fipa_acl_message_s* acl;
+  char* id;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  acl = Ch_VaArg(interp, ap, struct fipa_acl_message_s*);
+  id = Ch_VaArg(interp, ap, char*);
+  retval = MC_AclSetConversationID(acl, id);
+  Ch_VaEnd(interp, ap);
+  return retval;
+}
+
 /* MC_AclSetPerformative */
 EXPORTCH int
 MC_AclSetPerformative_chdl(void* varg)
@@ -2288,6 +2488,7 @@ MC_ComposeAgent_chdl(void *varg) /*{{{*/
   const char* code;
   const char* return_var_name;
   const char* server;
+  const char* workgroup_code;
   int persistent;
 
   ChInterp_t interp;
@@ -2301,6 +2502,7 @@ MC_ComposeAgent_chdl(void *varg) /*{{{*/
   code = Ch_VaArg(interp, ap, const char*);
   return_var_name = Ch_VaArg(interp, ap, const char*);
   server = Ch_VaArg(interp, ap, const char*);
+  workgroup_code = Ch_VaArg(interp, ap, const char*);
   persistent = Ch_VaArg(interp, ap, int);
 
   retval= MC_ComposeAgent(
@@ -2310,6 +2512,7 @@ MC_ComposeAgent_chdl(void *varg) /*{{{*/
       code,
       return_var_name,
       server,
+			workgroup_code,
       persistent);
   Ch_VaEnd(interp, ap);
   return retval;
@@ -2390,15 +2593,42 @@ MC_DeleteAgent_chdl(void *varg) /*{{{*/
   ChInterp_t interp;
   ChVaList_t ap;
   MCAgent_t agent;
+  const char* agentName;
   int retval;
 
   Ch_VaStart(interp, ap, varg);
-  agent = Ch_VaArg(interp, ap, MCAgent_t);
+  agentName = Ch_VaArg(interp, ap, const char*);
+  agent = MC_FindAgentByName(NULL, agentName);
+  if (agent == NULL) {
+    return MC_ERR_NOT_FOUND;
+  }
   retval = MC_DeleteAgent(agent);
   Ch_VaEnd(interp, ap);
   return retval;
 } /*}}}*/
 
+EXPORTCH int
+MC_DeleteAgentWG_chdl(void *varg) /*{{{*/
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  const char* agentName;
+  MCAgent_t calling_agent;
+  MCAgent_t agent;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  calling_agent = Ch_VaArg(interp, ap, MCAgent_t);
+  agentName = Ch_VaArg(interp, ap, const char*);
+  agent = MC_FindAgentByName(NULL, agentName);
+  if (agent == NULL) {
+    return MC_ERR_NOT_FOUND;
+  }
+  retval = MC_DeleteAgentWG(calling_agent, agent);
+  Ch_VaEnd(interp, ap);
+  return retval;
+} /*}}}*/
+
 EXPORTCH int
 MC_DestroyServiceSearchResult_chdl(void* varg)
 {
@@ -3108,13 +3338,40 @@ MC_TerminateAgent_chdl(void *varg) /*{{{*/
 {
   ChInterp_t interp;
   ChVaList_t ap;
+  const char* agentName;
   MCAgent_t agent;
   int retval;
 
   Ch_VaStart(interp, ap, varg);
-  agent = Ch_VaArg(interp, ap, MCAgent_t);
+  agentName = Ch_VaArg(interp, ap, const char*);
+  agent = MC_FindAgentByName(NULL, agentName);
+  if (agent == NULL) {
+    return MC_ERR_NOT_FOUND;
+  }
   retval = MC_TerminateAgent(agent);
   Ch_VaEnd(interp, ap);
   return retval;
 } /*}}}*/
 
+/* MC_TerminateAgentWG_chdl */
+EXPORTCH int
+MC_TerminateAgentWG_chdl(void *varg) /*{{{*/
+{
+  ChInterp_t interp;
+  ChVaList_t ap;
+  const char* agentName;
+  MCAgent_t calling_agent;
+  MCAgent_t agent;
+  int retval;
+
+  Ch_VaStart(interp, ap, varg);
+  calling_agent = Ch_VaArg(interp, ap, MCAgent_t);
+  agentName = Ch_VaArg(interp, ap, MCAgent_t);
+  agent = MC_FindAgentByName(NULL, agentName);
+  if (agent == NULL) {
+    return MC_ERR_NOT_FOUND;
+  }
+  retval = MC_TerminateAgentWG(calling_agent, agent);
+  Ch_VaEnd(interp, ap);
+  return retval;
+} /*}}}*/
