@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -40,6 +40,7 @@
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <netdb.h>
+#include <errno.h>
 #include "config.h"
 #else
 #include "winconfig.h"
@@ -53,6 +54,7 @@
 #include "include/xml_compose.h"
 #include "include/xml_helper.h"
 #include "include/xml_parser.h"
+#include "include/macros.h"
 
 #include "security/asm_node.h"
 
@@ -167,6 +169,7 @@ message_InitializeFromAgent(
   }
   message->agent_xml_flag = 0;
   message->target = strdup("ams");
+	message->sending_agent_name = strdup(agent->name);
   /* Set up message->addr */
   buf = (char*)malloc
     (
@@ -222,12 +225,13 @@ message_InitializeFromConnection(
   /* Receive the message */
   while(1) {
 #ifndef _WIN32
-    n = recvfrom(connection->clientfd,
+    /*n = recvfrom(connection->clientfd,
         (void *) buffer,
         (size_t) sizeof(char)*SOCKET_INPUT_SIZE,
         0,
         (struct sockaddr *) 0,
-        (socklen_t *) 0);
+        (socklen_t *) 0); */
+		n = recv(connection->clientfd, (void*)buffer, (size_t) sizeof(char)*SOCKET_INPUT_SIZE, MSG_WAITALL);
 #else
     n = recvfrom(connection->clientfd,
         (void *) buffer,
@@ -238,6 +242,7 @@ message_InitializeFromConnection(
 #endif
     if (n < 0) {
       free(buffer);
+			SOCKET_ERROR();
       return MC_ERR_CONNECT;
     } 
     else if (n == 0) {
@@ -553,19 +558,68 @@ auth_rece_send_msg(int sockfd, char *hostname, char *message, char *privkey, cha
   return ret;
 }
 
+#define MSG_THREADS 40
 int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey)
 {
+	THREAD_T msg_thread;
+	message_send_arg_t* arg;
+#ifndef _WIN32
+	pthread_attr_t attr;
+	pthread_attr_init(&attr);
+#else
+	int stack_size = 0;
+#endif
+
+	arg = (message_send_arg_t*)malloc(sizeof(message_send_arg_t));
+	arg->mc_platform = mc_platform;
+	arg->message = message;
+	arg->privatekey = privatekey;
+
+	MUTEX_LOCK(&mc_platform->acc->msg_thread_lock);
+	while(mc_platform->acc->num_msg_threads >= MSG_THREADS) {
+		COND_WAIT(&mc_platform->acc->msg_thread_cond, &mc_platform->acc->msg_thread_lock);
+	}
+	mc_platform->acc->num_msg_threads++;
+	MUTEX_UNLOCK(&mc_platform->acc->msg_thread_lock);
+	/* DEBUG */
+	THREAD_CREATE(&msg_thread, message_send_Thread, arg);
+	THREAD_DETACH(msg_thread);
+}
+
+
+#define MSG_THREAD_EXIT() \
+	free(arg); \
+	message_Destroy(message); \
+	MUTEX_LOCK(&mc_platform->acc->msg_thread_lock); \
+	mc_platform->acc->num_msg_threads--; \
+	COND_SIGNAL(&mc_platform->acc->msg_thread_cond); \
+	MUTEX_UNLOCK(&mc_platform->acc->msg_thread_lock); \
+	THREAD_EXIT()
+
+#ifndef _WIN32
+  void*
+message_send_Thread(void* arg)
+#else
+  DWORD WINAPI
+message_send_Thread( LPVOID arg )
+#endif
+{
+	mc_platform_t* mc_platform;
+	message_p message;
+	char* privatekey;
   char *buffer;
   mtp_http_t* mtp_http;
   int n, ret;
 #ifndef _WIN32
   int skt;
   struct sockaddr_in sktin;
+  struct addrinfo* myaddrinfo;
 #else
   SOCKET skt;
   SOCKADDR_IN sktin;
 #endif
-  struct hostent *host;
+  struct hostent host;
+  struct hostent *host_result;
   char *buf;
   char *hostname;
 #ifndef _WIN32
@@ -573,6 +627,19 @@ int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey
 #endif
   int port;
 
+	int num_tries = 0;
+	const int max_num_tries = 100;
+	int errnum;
+
+	dynstring_t* message_string;
+
+	char hostbuf[80];
+	int hostbuf_len = 80;
+
+	mc_platform = ((message_send_arg_t*)arg)->mc_platform;
+	message = ((message_send_arg_t*)arg)->message;
+	privatekey = ((message_send_arg_t*)arg)->privatekey;
+		
   /* Compose the http message */
   if (
       mtp_http_ComposeMessage(
@@ -580,7 +647,8 @@ int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey
         )
      )
   {
-    return MC_ERR;
+		fprintf(stderr, "Compose Message Error. %s:%d\n", __FILE__, __LINE__);
+    MSG_THREAD_EXIT();
   }
 
   /* We need to split up the address into a hostname and port. */
@@ -592,32 +660,58 @@ int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey
   if((skt = socket(PF_INET, SOCK_STREAM, 0)) < 0) 
   { 
     fprintf(stderr, "Error - can't create socket\n");
-    return -1;
+    MSG_THREAD_EXIT();
   }
 
   memset(&sktin, 0, sizeof(sktin));
   sktin.sin_family = PF_INET;
   sktin.sin_port = htons(port);
 
-  if((host = gethostbyname(hostname))) 
+#ifndef _WIN32
+  //if(gethostbyname_r(hostname, &host, hostbuf, hostbuf_len, &host_result, &errnum)) 
+  if( (errnum = getaddrinfo(hostname, NULL, NULL, &myaddrinfo)) == 0 )
+#else
+  if(host_result = gethostbyname(hostname)) 
+#endif
   {
-    memcpy(&sktin.sin_addr, host->h_addr, host->h_length);
+#ifdef _WIN32
+    host = *host_result;
+	memcpy(&sktin.sin_addr, host.h_addr, host.h_length);
+#else
+    ((struct sockaddr_in*)myaddrinfo->ai_addr)->sin_port = htons(port);
+#endif
   }
   else if((sktin.sin_addr.s_addr = inet_addr(hostname)) < 0) 
   {
     fprintf(stderr, "Error - can't get host entry for %s\n", hostname);
     free(buf);
-    return -1;
-  }
+    MSG_THREAD_EXIT();
+  } 
 
-  if(connect(skt, (struct sockaddr *) &sktin, sizeof(sktin)) < 0) {
+  while(
+#ifndef _WIN32
+		(connect(skt, myaddrinfo->ai_addr, sizeof(struct sockaddr)) < 0) &&
+#else
+        (connect(skt, (struct sockaddr *) &sktin, sizeof(sktin)) < 0) &&
+#endif
+		(num_tries < max_num_tries)
+		) {
+		printf("%s ERROR Socket Connect failed... errno:%s\n", message->sending_agent_name, strerror(errno));
+#ifndef _WIN32
+		usleep(100000);
+#else
+		Sleep(100);
+#endif
+		num_tries++;
+  }
+	if (num_tries == max_num_tries) {
     fprintf(stderr, "Error - can't connect to %s:%d\n",
         hostname,
         port
         );
     free(buf);
-    return MC_ERR_CONNECT;
-  }
+    MSG_THREAD_EXIT();
+	}
 
 #ifdef NEW_SECURITY 
   /*  authenticate the receiver of message */
@@ -625,28 +719,32 @@ int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey
   if( ret == 1  ){
     printf("Successfull Authenticate and but send of MA is failed \n");
 #ifndef _WIN32
-    close(skt);
+    if(close(skt) < 0) {
+			SOCKET_ERROR();
+		}
 #else
     closesocket(skt);
 #endif
     free(buf);
-    return MC_ERR_SEND;
+		fprintf(stderr, "Security Error. %s:%d\n", __FILE__, __LINE__);
+    MSG_THREAD_EXIT();
   }else if(ret != 2){
 #ifndef _WIN32
-    close(skt);
+    if(close(skt) < 0) {
+			SOCKET_ERROR();
+		}
 #else
     closesocket(skt);
 #endif
     free(buf);
-    return MC_ERR_SEND;
+		fprintf(stderr, "Security Error. %s:%d\n", __FILE__, __LINE__);
+    MSG_THREAD_EXIT();
   }else if(ret == 2) // successfully authenticate and send
     //printf("successfully authenticate and send \n");        
 #else
-
-    //printf("sending message body --\n%s\n--", message->message_body);	
     if(send(skt, message->message_body, strlen(message->message_body), 0) < 0) {
       perror(send);
-      printf("cannot write to socket %s:%d\n");
+      printf("cannot write to socket %s:%d\n", __FILE__, __LINE__);
     }
     else
 #endif /* NEW_SECURITY */
@@ -655,25 +753,43 @@ int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey
       buffer = (char*) malloc(sizeof(char) * (SOCKET_INPUT_SIZE + 1));
       CHECK_NULL(buffer, exit(0););
       mtp_http = mtp_http_New();
+			message_string = dynstring_New();
+			while(1) {
 #ifndef _WIN32
-      n = recvfrom(skt,
-          (void *) buffer,
-          (size_t) sizeof(char)*SOCKET_INPUT_SIZE,
-          0,
-          (struct sockaddr *) 0,
-          (socklen_t *) 0);
+/*				n = recvfrom(skt,
+						(void *) buffer,
+						(size_t) sizeof(char)*SOCKET_INPUT_SIZE,
+						0,
+						(struct sockaddr *) 0,
+						(socklen_t *) 0);
+*/
+				n = recv(skt, (void*)buffer, (size_t) sizeof(char)*SOCKET_INPUT_SIZE, MSG_WAITALL);
 #else
-      n = recvfrom(skt,
-          (void *) buffer,
-          (size_t) sizeof(char)*SOCKET_INPUT_SIZE,
-          0,
-          (struct sockaddr *) 0,
-          0);
+				n = recvfrom(skt,
+						(void *) buffer,
+						(size_t) sizeof(char)*SOCKET_INPUT_SIZE,
+						0,
+						(struct sockaddr *) 0,
+						0);
 #endif
-      if( mtp_http_Parse(mtp_http, buffer) ) {
+				if (n<0) {
+					/* There was an error receiving the response. */
+					SOCKET_ERROR();
+					free(buffer);
+					MSG_THREAD_EXIT();
+				}
+				else if (n==0) {
+					break;
+				}
+				else
+				{
+					dynstring_Append(message_string, buffer);
+				}
+			}
+      if( mtp_http_Parse(mtp_http, message_string->message) ) {
         fprintf(stderr, "http parsing error: Response expected. %s:%d\n",
             __FILE__, __LINE__);
-        fprintf(stderr, "Received message was:\n%s\n", buffer);
+        fprintf(stderr, "Received message was:\n%s\n", message_string->message);
       }
       if (mtp_http->response_code != 200) {
         fprintf(stderr, "Warning: remote http server responded: %d %s\n",
@@ -684,13 +800,16 @@ int message_Send(mc_platform_t* mc_platform, message_p message, char *privatekey
     }
 
 #ifndef _WIN32
-  close(skt);
+  if(close(skt) <0) {
+		SOCKET_ERROR();
+	}
 #else
   closesocket(skt);
 #endif
   free(buf);
   free(buffer);
-  return 0;
+	dynstring_Destroy(message_string);
+  MSG_THREAD_EXIT();
 }
 
-
+#undef MSG_THREAD_EXIT()
