@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -38,6 +38,7 @@
 #include <time.h>
 #ifndef _WIN32
 #include <unistd.h>
+#include <errno.h>
 #include "config.h"
 #else
 #include "winconfig.h"
@@ -203,13 +204,14 @@ mtp_http_InitializeFromConnection
     }
 #else
 	#ifndef _WIN32
-	n = recvfrom(connection->clientfd,
+/*	n = recvfrom(connection->clientfd,
 		(void *) buffer,
 		(size_t) sizeof(char)*SOCKET_INPUT_SIZE,
 		0,
 		(struct sockaddr *) 0,
-		(socklen_t *) 0);
+		(socklen_t *) 0); */
 		//printf("recev message %s\n", buffer);	
+		n = recv(connection->clientfd, (void*)buffer, (size_t) sizeof(char)*SOCKET_INPUT_SIZE, 0);
 	#else
 	n = recvfrom(connection->clientfd,
 		(void *) buffer,
@@ -222,6 +224,7 @@ mtp_http_InitializeFromConnection
     received_len += n;
     if (n < 0) {
       free(buffer);
+			SOCKET_ERROR();
       return MC_ERR_CONNECT;
     } 
     else if (n == 0 || n < SOCKET_INPUT_SIZE) {
@@ -235,12 +238,14 @@ mtp_http_InitializeFromConnection
       if(mtp_http_ParseHeader(http_header, message_string->message) == NULL) {
         fprintf(stderr, "Error parsing HTTP Header. %s:%d\n",
             __FILE__, __LINE__);
+				fprintf(stderr, "Message was %s\n", message_string->message);
         dynstring_Destroy(message_string);
         mtp_http_Destroy(http_header);
         free(buffer);
         return 2;
       }
-      if ( received_len < (atoi(http_header->content_length) + http_header->header_length) ) {
+      if ( http_header->content_length != NULL &&
+				received_len < (atoi(http_header->content_length) + http_header->header_length) ) {
         mtp_http_Destroy(http_header);
         continue;
       }
@@ -279,18 +284,46 @@ mtp_http_InitializeFromConnection
       }
 
       if (n < SOCKET_INPUT_SIZE) {
-        buffer = strdup("HTTP/1.0 200 OK\r\nConnection: Close\r\n\r\nMobile C");
-        send(
-            connection->clientfd,
-            (void*)buffer,
-            sizeof(char)*strlen(buffer),
-            0 );
+				if (http->http_performative == HTTP_HEAD) {
+					buffer = (char*)malloc(sizeof(char) * 300);
+					if (buffer == NULL) {
+						fprintf(stderr, "Memory Error. %s:%d\n", __FILE__,__LINE__);
+					}
+					sprintf(buffer, "HTTP/1.0 200 OK\r\nContent-Length: %d\r\nConnection: close\r\nContent-Type:text/html\r\n\r\n%s", strlen(PACKAGE_STRING), PACKAGE_STRING);
+					if(send(
+							connection->clientfd,
+							(void*)buffer,
+							sizeof(char)*strlen(buffer),
+							0 ) < 0) {
+						SOCKET_ERROR();
+					}
 #ifdef _WIN32
-        closesocket(connection->clientfd);
+					closesocket(connection->clientfd);
 #else
-        close(connection->clientfd);
+					if(close(connection->clientfd) < 0) {
+						SOCKET_ERROR();
+					}
 #endif
-        free(buffer);
+					free(buffer);
+					return ERR;
+				} else {
+					buffer = strdup("HTTP/1.0 200 OK\r\nConnection: Close\r\n\r\nMobile C");
+					if(send(
+							connection->clientfd,
+							(void*)buffer,
+							sizeof(char)*strlen(buffer),
+							0 ) < 0 ) {
+						SOCKET_ERROR();
+					}
+#ifdef _WIN32
+					closesocket(connection->clientfd);
+#else
+					if(close(connection->clientfd) < 0) {
+						SOCKET_ERROR();
+					}
+#endif
+					free(buffer);
+				}
       }
       break;
     } else {
@@ -446,6 +479,7 @@ const char* mtp_http_ParseHeader(struct mtp_http_s* http, const char* string)
     );
   do
   {
+		if(strlen(line) == 0) {break;}
     line = http_GetExpression
       (
        line,
@@ -565,7 +599,7 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
     http->content = (struct mtp_http_content_s*)malloc(sizeof(struct mtp_http_content_s));
     memset(http->content, 0, sizeof(struct mtp_http_content_s));
     /* Copy rest of contents into the data member */
-    if (line != NULL) {
+    if (line != NULL && http->content_length > 0) {
       http->content->data = (void*)malloc
         (
          sizeof(char) * 
@@ -654,7 +688,8 @@ mtp_http_Parse(struct mtp_http_s* http, const char* string)
   if (
       (http->http_performative == HTTP_POST) ||
       (http->http_performative == HTTP_PUT) ||
-      (http->http_performative == HTTP_RESPONSE)
+      (http->http_performative == HTTP_RESPONSE) ||
+			(http->http_performative == HTTP_HEAD)
      )
     return 0;
   else
