@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -192,6 +192,17 @@ agent_xml_compose__agent_data(agent_p agent)
         );
   }
 
+  /* Add the 'wg_code' node */
+  tmp_node = agent_xml_compose__wg_code(agent);
+  if (tmp_node != NULL) {
+    mxmlAdd(
+        node,
+        MXML_ADD_AFTER,
+        NULL,
+        tmp_node
+        );
+  }
+
   /* Add the 'task' node */
   tmp_node = agent_xml_compose__tasks(agent);
   if (tmp_node != NULL) {
@@ -254,6 +265,22 @@ agent_xml_compose__home(agent_p agent)
   return node;
 }
 
+  mxml_node_t*
+agent_xml_compose__wg_code(agent_p agent)
+{
+  mxml_node_t* node;
+  node = mxmlNewElement(
+      NULL,
+      "WG_CODE"
+      );
+  mxmlNewText(
+      node,
+      0,
+      agent->wg_code
+      );
+  return node;
+}
+
   mxml_node_t*
 agent_xml_compose__tasks(agent_p agent)
 {
