@@ -1,6 +1,6 @@
 /* SVN FILE INFO
- * $Revision: 319 $ : Last Committed Revision
- * $Date: 2009-04-29 17:04:14 -0700 (Wed, 29 Apr 2009) $ : Last Committed Date */
+ * $Revision: 440 $ : Last Committed Revision
+ * $Date: 2010-01-06 16:36:35 -0800 (Wed, 06 Jan 2010) $ : Last Committed Date */
 /*[
  * Copyright (c) 2007 Integration Engineering Laboratory
                       University of California, Davis
@@ -140,6 +140,15 @@ agent_xml_parse__agent_data
     return err_code;
   }
 
+  xml_parser->node = xml_get_child(
+      agent_data_node,
+      "WG_CODE",
+      1
+      );
+  if ( (err_code = agent_xml_parse__wg_code(agent, xml_parser)) ) {
+    return err_code;
+  }
+
   xml_parser->node = xml_get_child(
       agent_data_node,
       "TASKS",
@@ -258,6 +267,37 @@ agent_xml_parse__sender(agent_p agent, xml_parser_p xml_parser)
   return MC_SUCCESS;
 }
 
+/* *** */
+/* agent_xml_parse__wg_code */
+error_code_t
+agent_xml_parse__wg_code(agent_p agent, xml_parser_p xml_parser)
+{
+  char *text;
+  const mxml_node_t* wg_code_node;
+  if (xml_parser->node == NULL) {
+    /* This is fine: wg_code field is optional */
+    agent->wg_code = NULL;
+    return MC_SUCCESS;
+  }
+  wg_code_node = xml_parser->node;
+  text = xml_get_text( wg_code_node );
+  /* If there is no text, just return... */
+	if (text == NULL) {
+		agent->wg_code=NULL;
+		return MC_SUCCESS;
+	}
+
+  agent->wg_code = (char*)malloc(
+      sizeof(char)*(strlen(text)+1)
+      );
+  strcpy(
+      agent->wg_code,
+      text
+      );
+  free(text);
+  return MC_SUCCESS;
+}
+
 /* *** */
 /* agent_xml_parse__tasks */
 error_code_t
